# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: weatherData.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'weatherData.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11weatherData.proto\x12\x0fWeatherSettings\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1cgoogle/protobuf/struct.proto\"\xa7\x05\n\x0eWeatherSetting\x12\x30\n\x0bweatherRows\x18\x01 \x03(\x0b\x32\x1b.WeatherSettings.WeatherRow\x12\x10\n\x08Latitude\x18\x02 \x01(\x02\x12\x11\n\tLongitude\x18\x03 \x01(\x02\x12\x18\n\x10\x45levation_Metres\x18\x04 \x01(\x02\x12-\n\tStartDate\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12+\n\x07\x45ndDate\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12=\n\x0fObservationType\x18\x07 \x01(\x0e\x32$.WeatherSettings.ObservationTypeEnum\x12\x33\n\nDataSource\x18\x08 \x01(\x0e\x32\x1f.WeatherSettings.DataSourceEnum\x12\x1e\n\x16HorizonShadowsIncluded\x18\t \x01(\x08\x12\x13\n\x0bRadiationDB\x18\n \x01(\t\x12\x0f\n\x07MeteoDB\x18\x0b \x01(\t\x12\x11\n\tHorizonDB\x18\x0c \x01(\t\x12G\n\x1fLocalStandardTimeOffset_Minutes\x18\r \x01(\x0b\x32\x1e.WeatherSettings.NullableInt32\x12\x1a\n\x12LocalTimeZone_IANA\x18\x0e \x01(\t\x12\x41\n\x11ObservationPeriod\x18\x0f \x01(\x0e\x32&.WeatherSettings.ObservationPeriodEnum\x12\x15\n\rversionNumber\x18\x10 \x01(\x05\x12<\n\x19ObservationPeriodDuration\x18\x11 \x01(\x0b\x32\x19.google.protobuf.Duration\"\xff\x08\n\nWeatherRow\x12\x37\n\x13ObservationTime_UTC\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12W\n/GlobalHorizontalIrradiance_WattsPerMetreSquared\x18\x02 \x01(\x0b\x32\x1e.WeatherSettings.NullableFloat\x12S\n+DirectNormalIrradiance_WattsPerMetreSquared\x18\x03 \x01(\x0b\x32\x1e.WeatherSettings.NullableFloat\x12X\n0DiffuseHorizontalIrradiance_WattsPerMetreSquared\x18\x04 \x01(\x0b\x32\x1e.WeatherSettings.NullableFloat\x12\x42\n\x1aSurfaceAirPressure_Pascals\x18\x05 \x01(\x0b\x32\x1e.WeatherSettings.NullableFloat\x12Q\n)DryBulbTemperature_DegreesCeliusAt2Metres\x18\x06 \x01(\x0b\x32\x1e.WeatherSettings.NullableFloat\x12N\n&WindVelocity_MetresPerSecondAt10Metres\x18\x07 \x01(\x0b\x32\x1e.WeatherSettings.NullableFloat\x12Y\n1WindDirection_DegreesClockwiseFromNorthAt10Metres\x18\x08 \x01(\x0b\x32\x1e.WeatherSettings.NullableFloat\x12@\n\x18RelativeHumidity_Percent\x18\t \x01(\x0b\x32\x1e.WeatherSettings.NullableFloat\x12V\n.SurfaceInfraredIrradiance_WattsPerMeterSquared\x18\n \x01(\x0b\x32\x1e.WeatherSettings.NullableFloat\x12\x41\n\x19PrecipitableWaterVapourCM\x18\x0b \x01(\x0b\x32\x1e.WeatherSettings.NullableFloat\x12\x33\n\x0bSolarZenith\x18\x0c \x01(\x0b\x32\x1e.WeatherSettings.NullableFloat\x12\x34\n\x0cSolarAzimuth\x18\r \x01(\x0b\x32\x1e.WeatherSettings.NullableFloat\x12\x33\n\x0bOzone_AtmCm\x18\x0e \x01(\x0b\x32\x1e.WeatherSettings.NullableFloat\x12\x31\n\tTurbidity\x18\x0f \x01(\x0b\x32\x1e.WeatherSettings.NullableFloat\x12>\n\x16\x46\x61rFieldAlbedo_Percent\x18\x10 \x01(\x0b\x32\x1e.WeatherSettings.NullableFloat\"S\n\rNullableFloat\x12*\n\x04null\x18\x01 \x01(\x0e\x32\x1a.google.protobuf.NullValueH\x00\x12\x0e\n\x04\x64\x61ta\x18\x02 \x01(\x02H\x00\x42\x06\n\x04kind\"S\n\rNullableInt32\x12*\n\x04null\x18\x01 \x01(\x0e\x32\x1a.google.protobuf.NullValueH\x00\x12\x0e\n\x04\x64\x61ta\x18\x02 \x01(\x05H\x00\x42\x06\n\x04kind*\'\n\x0e\x44\x61taSourceEnum\x12\n\n\x06UPLOAD\x10\x00\x12\t\n\x05PVGIS\x10\x01*,\n\x13ObservationTypeEnum\x12\x07\n\x03TMY\x10\x00\x12\x0c\n\x08OBSERVED\x10\x01*\x83\x01\n\x15ObservationPeriodEnum\x12%\n!Deprecated_ZenithAzimuthSpecified\x10\x00\x12\x11\n\rInstantaneous\x10\x01\x12\r\n\tEndOfHour\x10\x02\x12\x0f\n\x0bStartOfHour\x10\x03\x12\x10\n\x0cMiddleOfHour\x10\x04\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'weatherData_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_DATASOURCEENUM']._serialized_start=2139
  _globals['_DATASOURCEENUM']._serialized_end=2178
  _globals['_OBSERVATIONTYPEENUM']._serialized_start=2180
  _globals['_OBSERVATIONTYPEENUM']._serialized_end=2224
  _globals['_OBSERVATIONPERIODENUM']._serialized_start=2227
  _globals['_OBSERVATIONPERIODENUM']._serialized_end=2358
  _globals['_WEATHERSETTING']._serialized_start=134
  _globals['_WEATHERSETTING']._serialized_end=813
  _globals['_WEATHERROW']._serialized_start=816
  _globals['_WEATHERROW']._serialized_end=1967
  _globals['_NULLABLEFLOAT']._serialized_start=1969
  _globals['_NULLABLEFLOAT']._serialized_end=2052
  _globals['_NULLABLEINT32']._serialized_start=2054
  _globals['_NULLABLEINT32']._serialized_end=2137
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: uncertainty.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'uncertainty.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import uncertaintyMessages_pb2 as uncertaintyMessages__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11uncertainty.proto\x12\x11grpc.SunSolve.API\x1a\x19uncertaintyMessages.proto2u\n\x0bUncertainty\x12\x66\n\x13UncertaintyAnalysis\x12%.grpc.SunSolve.API.UncertaintyRequest\x1a&.grpc.SunSolve.API.UncertaintyResponse0\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'uncertainty_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_UNCERTAINTY']._serialized_start=67
  _globals['_UNCERTAINTY']._serialized_end=184
# @@protoc_insertion_point(module_scope)



import datetime

from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Iterable as _Iterable, Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class TrackingType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    NotSet: _ClassVar[TrackingType]
    SAT: _ClassVar[TrackingType]

class DistributionInput(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    GHI: _ClassVar[DistributionInput]
    DiffuseFraction: _ClassVar[DistributionInput]
    WindSpeed: _ClassVar[DistributionInput]
    Temperature: _ClassVar[DistributionInput]
    ModulePower: _ClassVar[DistributionInput]
    SpectralCorrection: _ClassVar[DistributionInput]
    SoilingFront: _ClassVar[DistributionInput]
    SoilingRear: _ClassVar[DistributionInput]
    Uc: _ClassVar[DistributionInput]
    Uv: _ClassVar[DistributionInput]
    Alpha: _ClassVar[DistributionInput]
    AnnualDegradationRate: _ClassVar[DistributionInput]
    Availability: _ClassVar[DistributionInput]
    YieldModifier: _ClassVar[DistributionInput]
    CircumsolarFraction: _ClassVar[DistributionInput]
    UndulatingGround: _ClassVar[DistributionInput]
    ExtraIrradiance: _ClassVar[DistributionInput]
    Curtailment: _ClassVar[DistributionInput]
    DCHealth: _ClassVar[DistributionInput]
    InverterToInverterMismatch: _ClassVar[DistributionInput]
    StringToStringMismatch: _ClassVar[DistributionInput]
    ModuleToModuleMismatch: _ClassVar[DistributionInput]
    CellToCellMismatch: _ClassVar[DistributionInput]
    InverterEfficiency: _ClassVar[DistributionInput]
    ModulePowerTemperatureCoefficient: _ClassVar[DistributionInput]
    Albedo: _ClassVar[DistributionInput]
    RearStructuralShadingFactor: _ClassVar[DistributionInput]
    RearTransmissionFactor: _ClassVar[DistributionInput]

class DistributionType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    Constant: _ClassVar[DistributionType]
    Gaussian: _ClassVar[DistributionType]
    Weibull: _ClassVar[DistributionType]
    SkewedGaussian: _ClassVar[DistributionType]
    Arbitrary: _ClassVar[DistributionType]
NotSet: TrackingType
SAT: TrackingType
GHI: DistributionInput
DiffuseFraction: DistributionInput
WindSpeed: DistributionInput
Temperature: DistributionInput
ModulePower: DistributionInput
SpectralCorrection: DistributionInput
SoilingFront: DistributionInput
SoilingRear: DistributionInput
Uc: DistributionInput
Uv: DistributionInput
Alpha: DistributionInput
AnnualDegradationRate: DistributionInput
Availability: DistributionInput
YieldModifier: DistributionInput
CircumsolarFraction: DistributionInput
UndulatingGround: DistributionInput
ExtraIrradiance: DistributionInput
Curtailment: DistributionInput
DCHealth: DistributionInput
InverterToInverterMismatch: DistributionInput
StringToStringMismatch: DistributionInput
ModuleToModuleMismatch: DistributionInput
CellToCellMismatch: DistributionInput
InverterEfficiency: DistributionInput
ModulePowerTemperatureCoefficient: DistributionInput
Albedo: DistributionInput
RearStructuralShadingFactor: DistributionInput
RearTransmissionFactor: DistributionInput
Constant: DistributionType
Gaussian: DistributionType
Weibull: DistributionType
SkewedGaussian: DistributionType
Arbitrary: DistributionType

class UncertaintyResponse(_message.Message):
    __slots__ = ("pctComplete", "SimulationResult", "Summary", "InputsUsed")
    PCTCOMPLETE_FIELD_NUMBER: _ClassVar[int]
    SIMULATIONRESULT_FIELD_NUMBER: _ClassVar[int]
    SUMMARY_FIELD_NUMBER: _ClassVar[int]
    INPUTSUSED_FIELD_NUMBER: _ClassVar[int]
    pctComplete: float
    SimulationResult: UncertaintySimulationResult
    Summary: UncertaintySummary
    InputsUsed: UncertaintyUsedInputs
    def __init__(self, pctComplete: _Optional[float] = ..., SimulationResult: _Optional[_Union[UncertaintySimulationResult, _Mapping]] = ..., Summary: _Optional[_Union[UncertaintySummary, _Mapping]] = ..., InputsUsed: _Optional[_Union[UncertaintyUsedInputs, _Mapping]] = ...) -> None: ...

class UncertaintySimulationResult(_message.Message):
    __slots__ = ("SimulationIndex", "YearNumber", "TotalDCYield", "TotalACYield", "YearlyAvailableYieldAC", "Degradation")
    SIMULATIONINDEX_FIELD_NUMBER: _ClassVar[int]
    YEARNUMBER_FIELD_NUMBER: _ClassVar[int]
    TOTALDCYIELD_FIELD_NUMBER: _ClassVar[int]
    TOTALACYIELD_FIELD_NUMBER: _ClassVar[int]
    YEARLYAVAILABLEYIELDAC_FIELD_NUMBER: _ClassVar[int]
    DEGRADATION_FIELD_NUMBER: _ClassVar[int]
    SimulationIndex: float
    YearNumber: float
    TotalDCYield: float
    TotalACYield: float
    YearlyAvailableYieldAC: float
    Degradation: float
    def __init__(self, SimulationIndex: _Optional[float] = ..., YearNumber: _Optional[float] = ..., TotalDCYield: _Optional[float] = ..., TotalACYield: _Optional[float] = ..., YearlyAvailableYieldAC: _Optional[float] = ..., Degradation: _Optional[float] = ...) -> None: ...

class UncertaintySummary(_message.Message):
    __slots__ = ("YearlyPValue", "YearlyHistogram", "YearlyP50Deviation")
    YEARLYPVALUE_FIELD_NUMBER: _ClassVar[int]
    YEARLYHISTOGRAM_FIELD_NUMBER: _ClassVar[int]
    YEARLYP50DEVIATION_FIELD_NUMBER: _ClassVar[int]
    YearlyPValue: _containers.RepeatedCompositeFieldContainer[PValue]
    YearlyHistogram: _containers.RepeatedCompositeFieldContainer[Histogram]
    YearlyP50Deviation: _containers.RepeatedCompositeFieldContainer[P50Deviation]
    def __init__(self, YearlyPValue: _Optional[_Iterable[_Union[PValue, _Mapping]]] = ..., YearlyHistogram: _Optional[_Iterable[_Union[Histogram, _Mapping]]] = ..., YearlyP50Deviation: _Optional[_Iterable[_Union[P50Deviation, _Mapping]]] = ...) -> None: ...

class UncertaintyUsedInputs(_message.Message):
    __slots__ = ("Module", "System", "Electrical", "Optical", "Thermal", "Operation", "SimulationOptions", "ResultOptions")
    MODULE_FIELD_NUMBER: _ClassVar[int]
    SYSTEM_FIELD_NUMBER: _ClassVar[int]
    ELECTRICAL_FIELD_NUMBER: _ClassVar[int]
    OPTICAL_FIELD_NUMBER: _ClassVar[int]
    THERMAL_FIELD_NUMBER: _ClassVar[int]
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    SIMULATIONOPTIONS_FIELD_NUMBER: _ClassVar[int]
    RESULTOPTIONS_FIELD_NUMBER: _ClassVar[int]
    Module: ModuleInfo
    System: SystemInfo
    Electrical: ElectricalSettings
    Optical: OpticalSettings
    Thermal: ThermalSettings
    Operation: OperationalSettings
    SimulationOptions: UncertaintySimulationOptions
    ResultOptions: ResultOptions
    def __init__(self, Module: _Optional[_Union[ModuleInfo, _Mapping]] = ..., System: _Optional[_Union[SystemInfo, _Mapping]] = ..., Electrical: _Optional[_Union[ElectricalSettings, _Mapping]] = ..., Optical: _Optional[_Union[OpticalSettings, _Mapping]] = ..., Thermal: _Optional[_Union[ThermalSettings, _Mapping]] = ..., Operation: _Optional[_Union[OperationalSettings, _Mapping]] = ..., SimulationOptions: _Optional[_Union[UncertaintySimulationOptions, _Mapping]] = ..., ResultOptions: _Optional[_Union[ResultOptions, _Mapping]] = ...) -> None: ...

class PValue(_message.Message):
    __slots__ = ("Year", "P", "Value", "P50Deviation", "Conf95")
    YEAR_FIELD_NUMBER: _ClassVar[int]
    P_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    P50DEVIATION_FIELD_NUMBER: _ClassVar[int]
    CONF95_FIELD_NUMBER: _ClassVar[int]
    Year: int
    P: int
    Value: float
    P50Deviation: float
    Conf95: float
    def __init__(self, Year: _Optional[int] = ..., P: _Optional[int] = ..., Value: _Optional[float] = ..., P50Deviation: _Optional[float] = ..., Conf95: _Optional[float] = ...) -> None: ...

class Histogram(_message.Message):
    __slots__ = ("Year", "bins")
    YEAR_FIELD_NUMBER: _ClassVar[int]
    BINS_FIELD_NUMBER: _ClassVar[int]
    Year: int
    bins: _containers.RepeatedScalarFieldContainer[int]
    def __init__(self, Year: _Optional[int] = ..., bins: _Optional[_Iterable[int]] = ...) -> None: ...

class P50Deviation(_message.Message):
    __slots__ = ("Year", "Value")
    YEAR_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    Year: int
    Value: float
    def __init__(self, Year: _Optional[int] = ..., Value: _Optional[float] = ...) -> None: ...

class UncertaintyRequest(_message.Message):
    __slots__ = ("TMYDataSet", "ActualYearlyDataSet", "Module", "System", "Electrical", "Optical", "Thermal", "Operation", "SimulationOptions", "ResultOptions", "Distributions")
    TMYDATASET_FIELD_NUMBER: _ClassVar[int]
    ACTUALYEARLYDATASET_FIELD_NUMBER: _ClassVar[int]
    MODULE_FIELD_NUMBER: _ClassVar[int]
    SYSTEM_FIELD_NUMBER: _ClassVar[int]
    ELECTRICAL_FIELD_NUMBER: _ClassVar[int]
    OPTICAL_FIELD_NUMBER: _ClassVar[int]
    THERMAL_FIELD_NUMBER: _ClassVar[int]
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    SIMULATIONOPTIONS_FIELD_NUMBER: _ClassVar[int]
    RESULTOPTIONS_FIELD_NUMBER: _ClassVar[int]
    DISTRIBUTIONS_FIELD_NUMBER: _ClassVar[int]
    TMYDataSet: TMYTimeStepDataSet
    ActualYearlyDataSet: ActualTimeStepDataSet
    Module: ModuleInfo
    System: SystemInfo
    Electrical: ElectricalSettings
    Optical: OpticalSettings
    Thermal: ThermalSettings
    Operation: OperationalSettings
    SimulationOptions: UncertaintySimulationOptions
    ResultOptions: ResultOptions
    Distributions: _containers.RepeatedCompositeFieldContainer[Distribution]
    def __init__(self, TMYDataSet: _Optional[_Union[TMYTimeStepDataSet, _Mapping]] = ..., ActualYearlyDataSet: _Optional[_Union[ActualTimeStepDataSet, _Mapping]] = ..., Module: _Optional[_Union[ModuleInfo, _Mapping]] = ..., System: _Optional[_Union[SystemInfo, _Mapping]] = ..., Electrical: _Optional[_Union[ElectricalSettings, _Mapping]] = ..., Optical: _Optional[_Union[OpticalSettings, _Mapping]] = ..., Thermal: _Optional[_Union[ThermalSettings, _Mapping]] = ..., Operation: _Optional[_Union[OperationalSettings, _Mapping]] = ..., SimulationOptions: _Optional[_Union[UncertaintySimulationOptions, _Mapping]] = ..., ResultOptions: _Optional[_Union[ResultOptions, _Mapping]] = ..., Distributions: _Optional[_Iterable[_Union[Distribution, _Mapping]]] = ...) -> None: ...

class TMYTimeStepDataSet(_message.Message):
    __slots__ = ("TimeStepDataPoints",)
    TIMESTEPDATAPOINTS_FIELD_NUMBER: _ClassVar[int]
    TimeStepDataPoints: _containers.RepeatedCompositeFieldContainer[TimeStepData]
    def __init__(self, TimeStepDataPoints: _Optional[_Iterable[_Union[TimeStepData, _Mapping]]] = ...) -> None: ...

class ActualTimeStepDataSet(_message.Message):
    __slots__ = ("TimeStepDataPoints",)
    TIMESTEPDATAPOINTS_FIELD_NUMBER: _ClassVar[int]
    TimeStepDataPoints: _containers.RepeatedCompositeFieldContainer[ActualTimeStepDataForYear]
    def __init__(self, TimeStepDataPoints: _Optional[_Iterable[_Union[ActualTimeStepDataForYear, _Mapping]]] = ...) -> None: ...

class ActualTimeStepDataForYear(_message.Message):
    __slots__ = ("YearIndex", "TimeStepDataPoints")
    YEARINDEX_FIELD_NUMBER: _ClassVar[int]
    TIMESTEPDATAPOINTS_FIELD_NUMBER: _ClassVar[int]
    YearIndex: int
    TimeStepDataPoints: _containers.RepeatedCompositeFieldContainer[TimeStepData]
    def __init__(self, YearIndex: _Optional[int] = ..., TimeStepDataPoints: _Optional[_Iterable[_Union[TimeStepData, _Mapping]]] = ...) -> None: ...

class TimeStepData(_message.Message):
    __slots__ = ("EndOfPeriodUTC", "SolarPos", "Weather", "TiltAngleInDegrees", "Albedo", "SoilingFrontRate", "SoilingRearRate", "RearTransmissionFactor", "RearStructuralShadingFactor")
    ENDOFPERIODUTC_FIELD_NUMBER: _ClassVar[int]
    SOLARPOS_FIELD_NUMBER: _ClassVar[int]
    WEATHER_FIELD_NUMBER: _ClassVar[int]
    TILTANGLEINDEGREES_FIELD_NUMBER: _ClassVar[int]
    ALBEDO_FIELD_NUMBER: _ClassVar[int]
    SOILINGFRONTRATE_FIELD_NUMBER: _ClassVar[int]
    SOILINGREARRATE_FIELD_NUMBER: _ClassVar[int]
    REARTRANSMISSIONFACTOR_FIELD_NUMBER: _ClassVar[int]
    REARSTRUCTURALSHADINGFACTOR_FIELD_NUMBER: _ClassVar[int]
    EndOfPeriodUTC: _timestamp_pb2.Timestamp
    SolarPos: YieldTimeSeriesSolarPosition
    Weather: YieldTimeSeriesWeather
    TiltAngleInDegrees: float
    Albedo: float
    SoilingFrontRate: float
    SoilingRearRate: float
    RearTransmissionFactor: float
    RearStructuralShadingFactor: float
    def __init__(self, EndOfPeriodUTC: _Optional[_Union[datetime.datetime, _timestamp_pb2.Timestamp, _Mapping]] = ..., SolarPos: _Optional[_Union[YieldTimeSeriesSolarPosition, _Mapping]] = ..., Weather: _Optional[_Union[YieldTimeSeriesWeather, _Mapping]] = ..., TiltAngleInDegrees: _Optional[float] = ..., Albedo: _Optional[float] = ..., SoilingFrontRate: _Optional[float] = ..., SoilingRearRate: _Optional[float] = ..., RearTransmissionFactor: _Optional[float] = ..., RearStructuralShadingFactor: _Optional[float] = ...) -> None: ...

class ModuleInfo(_message.Message):
    __slots__ = ("LengthInM", "WidthInM", "CellToCellMismatch", "ModulePowerTemperatureCoefficient", "PowerRatingAtSTCInW", "Bifaciality")
    LENGTHINM_FIELD_NUMBER: _ClassVar[int]
    WIDTHINM_FIELD_NUMBER: _ClassVar[int]
    CELLTOCELLMISMATCH_FIELD_NUMBER: _ClassVar[int]
    MODULEPOWERTEMPERATURECOEFFICIENT_FIELD_NUMBER: _ClassVar[int]
    POWERRATINGATSTCINW_FIELD_NUMBER: _ClassVar[int]
    BIFACIALITY_FIELD_NUMBER: _ClassVar[int]
    LengthInM: float
    WidthInM: float
    CellToCellMismatch: float
    ModulePowerTemperatureCoefficient: float
    PowerRatingAtSTCInW: float
    Bifaciality: float
    def __init__(self, LengthInM: _Optional[float] = ..., WidthInM: _Optional[float] = ..., CellToCellMismatch: _Optional[float] = ..., ModulePowerTemperatureCoefficient: _Optional[float] = ..., PowerRatingAtSTCInW: _Optional[float] = ..., Bifaciality: _Optional[float] = ...) -> None: ...

class SystemInfo(_message.Message):
    __slots__ = ("NumberOfInverters", "StringsPerInverter", "ModulesPerString", "RowPitchInM", "ModuleAzimuthInDegrees", "FallbackModuleTiltInDegrees", "ModuleTracking", "TiltLimitInDegrees", "ModuleHeight")
    NUMBEROFINVERTERS_FIELD_NUMBER: _ClassVar[int]
    STRINGSPERINVERTER_FIELD_NUMBER: _ClassVar[int]
    MODULESPERSTRING_FIELD_NUMBER: _ClassVar[int]
    ROWPITCHINM_FIELD_NUMBER: _ClassVar[int]
    MODULEAZIMUTHINDEGREES_FIELD_NUMBER: _ClassVar[int]
    FALLBACKMODULETILTINDEGREES_FIELD_NUMBER: _ClassVar[int]
    MODULETRACKING_FIELD_NUMBER: _ClassVar[int]
    TILTLIMITINDEGREES_FIELD_NUMBER: _ClassVar[int]
    MODULEHEIGHT_FIELD_NUMBER: _ClassVar[int]
    NumberOfInverters: int
    StringsPerInverter: int
    ModulesPerString: int
    RowPitchInM: float
    ModuleAzimuthInDegrees: float
    FallbackModuleTiltInDegrees: float
    ModuleTracking: TrackingType
    TiltLimitInDegrees: float
    ModuleHeight: float
    def __init__(self, NumberOfInverters: _Optional[int] = ..., StringsPerInverter: _Optional[int] = ..., ModulesPerString: _Optional[int] = ..., RowPitchInM: _Optional[float] = ..., ModuleAzimuthInDegrees: _Optional[float] = ..., FallbackModuleTiltInDegrees: _Optional[float] = ..., ModuleTracking: _Optional[_Union[TrackingType, str]] = ..., TiltLimitInDegrees: _Optional[float] = ..., ModuleHeight: _Optional[float] = ...) -> None: ...

class OpticalSettings(_message.Message):
    __slots__ = ("BeamMultiplierFront", "IsotropicMultiplierFront", "BeamMultiplierRear", "IsotropicMultiplierRear", "FallbackAlbedo", "FallbackSoilingFront", "FallbackSoilingRear", "FallbackSpectralCorrection", "ExtraIrradiance", "FallbackRearTransmissionFactor", "FallbackRearStructuralShadingFactor")
    BEAMMULTIPLIERFRONT_FIELD_NUMBER: _ClassVar[int]
    ISOTROPICMULTIPLIERFRONT_FIELD_NUMBER: _ClassVar[int]
    BEAMMULTIPLIERREAR_FIELD_NUMBER: _ClassVar[int]
    ISOTROPICMULTIPLIERREAR_FIELD_NUMBER: _ClassVar[int]
    FALLBACKALBEDO_FIELD_NUMBER: _ClassVar[int]
    FALLBACKSOILINGFRONT_FIELD_NUMBER: _ClassVar[int]
    FALLBACKSOILINGREAR_FIELD_NUMBER: _ClassVar[int]
    FALLBACKSPECTRALCORRECTION_FIELD_NUMBER: _ClassVar[int]
    EXTRAIRRADIANCE_FIELD_NUMBER: _ClassVar[int]
    FALLBACKREARTRANSMISSIONFACTOR_FIELD_NUMBER: _ClassVar[int]
    FALLBACKREARSTRUCTURALSHADINGFACTOR_FIELD_NUMBER: _ClassVar[int]
    BeamMultiplierFront: float
    IsotropicMultiplierFront: float
    BeamMultiplierRear: float
    IsotropicMultiplierRear: float
    FallbackAlbedo: float
    FallbackSoilingFront: float
    FallbackSoilingRear: float
    FallbackSpectralCorrection: float
    ExtraIrradiance: float
    FallbackRearTransmissionFactor: float
    FallbackRearStructuralShadingFactor: float
    def __init__(self, BeamMultiplierFront: _Optional[float] = ..., IsotropicMultiplierFront: _Optional[float] = ..., BeamMultiplierRear: _Optional[float] = ..., IsotropicMultiplierRear: _Optional[float] = ..., FallbackAlbedo: _Optional[float] = ..., FallbackSoilingFront: _Optional[float] = ..., FallbackSoilingRear: _Optional[float] = ..., FallbackSpectralCorrection: _Optional[float] = ..., ExtraIrradiance: _Optional[float] = ..., FallbackRearTransmissionFactor: _Optional[float] = ..., FallbackRearStructuralShadingFactor: _Optional[float] = ...) -> None: ...

class ElectricalSettings(_message.Message):
    __slots__ = ("InverterEfficiency", "InverterToInverterMismatch", "StringToStringMismatch", "ModuleToModuleMismatch", "InverterWiringLoss", "StringWiringLoss", "MaxPowerTrackingLoss", "InverterClipping")
    INVERTEREFFICIENCY_FIELD_NUMBER: _ClassVar[int]
    INVERTERTOINVERTERMISMATCH_FIELD_NUMBER: _ClassVar[int]
    STRINGTOSTRINGMISMATCH_FIELD_NUMBER: _ClassVar[int]
    MODULETOMODULEMISMATCH_FIELD_NUMBER: _ClassVar[int]
    INVERTERWIRINGLOSS_FIELD_NUMBER: _ClassVar[int]
    STRINGWIRINGLOSS_FIELD_NUMBER: _ClassVar[int]
    MAXPOWERTRACKINGLOSS_FIELD_NUMBER: _ClassVar[int]
    INVERTERCLIPPING_FIELD_NUMBER: _ClassVar[int]
    InverterEfficiency: float
    InverterToInverterMismatch: float
    StringToStringMismatch: float
    ModuleToModuleMismatch: float
    InverterWiringLoss: float
    StringWiringLoss: float
    MaxPowerTrackingLoss: float
    InverterClipping: float
    def __init__(self, InverterEfficiency: _Optional[float] = ..., InverterToInverterMismatch: _Optional[float] = ..., StringToStringMismatch: _Optional[float] = ..., ModuleToModuleMismatch: _Optional[float] = ..., InverterWiringLoss: _Optional[float] = ..., StringWiringLoss: _Optional[float] = ..., MaxPowerTrackingLoss: _Optional[float] = ..., InverterClipping: _Optional[float] = ...) -> None: ...

class ThermalSettings(_message.Message):
    __slots__ = ("Uc", "Uv", "Alpha")
    UC_FIELD_NUMBER: _ClassVar[int]
    UV_FIELD_NUMBER: _ClassVar[int]
    ALPHA_FIELD_NUMBER: _ClassVar[int]
    Uc: float
    Uv: float
    Alpha: float
    def __init__(self, Uc: _Optional[float] = ..., Uv: _Optional[float] = ..., Alpha: _Optional[float] = ...) -> None: ...

class OperationalSettings(_message.Message):
    __slots__ = ("AnnualDegradationRate", "DCHealth", "Availability", "Curtailment", "YieldModifier", "UndulatingGround")
    ANNUALDEGRADATIONRATE_FIELD_NUMBER: _ClassVar[int]
    DCHEALTH_FIELD_NUMBER: _ClassVar[int]
    AVAILABILITY_FIELD_NUMBER: _ClassVar[int]
    CURTAILMENT_FIELD_NUMBER: _ClassVar[int]
    YIELDMODIFIER_FIELD_NUMBER: _ClassVar[int]
    UNDULATINGGROUND_FIELD_NUMBER: _ClassVar[int]
    AnnualDegradationRate: float
    DCHealth: float
    Availability: float
    Curtailment: float
    YieldModifier: float
    UndulatingGround: float
    def __init__(self, AnnualDegradationRate: _Optional[float] = ..., DCHealth: _Optional[float] = ..., Availability: _Optional[float] = ..., Curtailment: _Optional[float] = ..., YieldModifier: _Optional[float] = ..., UndulatingGround: _Optional[float] = ...) -> None: ...

class UncertaintySimulationOptions(_message.Message):
    __slots__ = ("NumberOfSimulations", "NumberOfYears")
    NUMBEROFSIMULATIONS_FIELD_NUMBER: _ClassVar[int]
    NUMBEROFYEARS_FIELD_NUMBER: _ClassVar[int]
    NumberOfSimulations: int
    NumberOfYears: int
    def __init__(self, NumberOfSimulations: _Optional[int] = ..., NumberOfYears: _Optional[int] = ...) -> None: ...

class ResultOptions(_message.Message):
    __slots__ = ("BinMin", "BinDelta", "PValues", "IncludeYieldResult")
    BINMIN_FIELD_NUMBER: _ClassVar[int]
    BINDELTA_FIELD_NUMBER: _ClassVar[int]
    PVALUES_FIELD_NUMBER: _ClassVar[int]
    INCLUDEYIELDRESULT_FIELD_NUMBER: _ClassVar[int]
    BinMin: float
    BinDelta: float
    PValues: _containers.RepeatedScalarFieldContainer[int]
    IncludeYieldResult: bool
    def __init__(self, BinMin: _Optional[float] = ..., BinDelta: _Optional[float] = ..., PValues: _Optional[_Iterable[int]] = ..., IncludeYieldResult: bool = ...) -> None: ...

class Distribution(_message.Message):
    __slots__ = ("Input", "SimToSimDistribution", "YrToYrDistribution", "StepToStepDistribution")
    INPUT_FIELD_NUMBER: _ClassVar[int]
    SIMTOSIMDISTRIBUTION_FIELD_NUMBER: _ClassVar[int]
    YRTOYRDISTRIBUTION_FIELD_NUMBER: _ClassVar[int]
    STEPTOSTEPDISTRIBUTION_FIELD_NUMBER: _ClassVar[int]
    Input: DistributionInput
    SimToSimDistribution: DistributionFunction
    YrToYrDistribution: DistributionFunction
    StepToStepDistribution: DistributionFunction
    def __init__(self, Input: _Optional[_Union[DistributionInput, str]] = ..., SimToSimDistribution: _Optional[_Union[DistributionFunction, _Mapping]] = ..., YrToYrDistribution: _Optional[_Union[DistributionFunction, _Mapping]] = ..., StepToStepDistribution: _Optional[_Union[DistributionFunction, _Mapping]] = ...) -> None: ...

class DistributionFunction(_message.Message):
    __slots__ = ("Type", "Constant", "Gaussian", "Weibull", "SkewedGaussian", "Arbitrary")
    TYPE_FIELD_NUMBER: _ClassVar[int]
    CONSTANT_FIELD_NUMBER: _ClassVar[int]
    GAUSSIAN_FIELD_NUMBER: _ClassVar[int]
    WEIBULL_FIELD_NUMBER: _ClassVar[int]
    SKEWEDGAUSSIAN_FIELD_NUMBER: _ClassVar[int]
    ARBITRARY_FIELD_NUMBER: _ClassVar[int]
    Type: DistributionType
    Constant: ConstantDistribution
    Gaussian: GaussianDistribution
    Weibull: WeibullDistribution
    SkewedGaussian: SkewedGaussianDistribution
    Arbitrary: ArbitraryDistribution
    def __init__(self, Type: _Optional[_Union[DistributionType, str]] = ..., Constant: _Optional[_Union[ConstantDistribution, _Mapping]] = ..., Gaussian: _Optional[_Union[GaussianDistribution, _Mapping]] = ..., Weibull: _Optional[_Union[WeibullDistribution, _Mapping]] = ..., SkewedGaussian: _Optional[_Union[SkewedGaussianDistribution, _Mapping]] = ..., Arbitrary: _Optional[_Union[ArbitraryDistribution, _Mapping]] = ...) -> None: ...

class ConstantDistribution(_message.Message):
    __slots__ = ("Value",)
    VALUE_FIELD_NUMBER: _ClassVar[int]
    Value: float
    def __init__(self, Value: _Optional[float] = ...) -> None: ...

class GaussianDistribution(_message.Message):
    __slots__ = ("x0", "sigma", "fmax", "numPointsInProbabilityFunction", "xLowerLimit", "xUpperLimit")
    X0_FIELD_NUMBER: _ClassVar[int]
    SIGMA_FIELD_NUMBER: _ClassVar[int]
    FMAX_FIELD_NUMBER: _ClassVar[int]
    NUMPOINTSINPROBABILITYFUNCTION_FIELD_NUMBER: _ClassVar[int]
    XLOWERLIMIT_FIELD_NUMBER: _ClassVar[int]
    XUPPERLIMIT_FIELD_NUMBER: _ClassVar[int]
    x0: float
    sigma: float
    fmax: float
    numPointsInProbabilityFunction: int
    xLowerLimit: float
    xUpperLimit: float
    def __init__(self, x0: _Optional[float] = ..., sigma: _Optional[float] = ..., fmax: _Optional[float] = ..., numPointsInProbabilityFunction: _Optional[int] = ..., xLowerLimit: _Optional[float] = ..., xUpperLimit: _Optional[float] = ...) -> None: ...

class SkewedGaussianDistribution(_message.Message):
    __slots__ = ("alpha", "zeta", "omega", "fmax", "numPointsInProbabilityFunction")
    ALPHA_FIELD_NUMBER: _ClassVar[int]
    ZETA_FIELD_NUMBER: _ClassVar[int]
    OMEGA_FIELD_NUMBER: _ClassVar[int]
    FMAX_FIELD_NUMBER: _ClassVar[int]
    NUMPOINTSINPROBABILITYFUNCTION_FIELD_NUMBER: _ClassVar[int]
    alpha: float
    zeta: float
    omega: float
    fmax: float
    numPointsInProbabilityFunction: int
    def __init__(self, alpha: _Optional[float] = ..., zeta: _Optional[float] = ..., omega: _Optional[float] = ..., fmax: _Optional[float] = ..., numPointsInProbabilityFunction: _Optional[int] = ...) -> None: ...

class WeibullDistribution(_message.Message):
    __slots__ = ("x0", "k", "hasPositivePolarity")
    X0_FIELD_NUMBER: _ClassVar[int]
    LAMBDA_FIELD_NUMBER: _ClassVar[int]
    K_FIELD_NUMBER: _ClassVar[int]
    HASPOSITIVEPOLARITY_FIELD_NUMBER: _ClassVar[int]
    x0: float
    k: float
    hasPositivePolarity: bool
    def __init__(self, x0: _Optional[float] = ..., k: _Optional[float] = ..., hasPositivePolarity: bool = ..., **kwargs) -> None: ...

class ArbitraryDistribution(_message.Message):
    __slots__ = ("x", "y")
    X_FIELD_NUMBER: _ClassVar[int]
    Y_FIELD_NUMBER: _ClassVar[int]
    x: _containers.RepeatedScalarFieldContainer[float]
    y: _containers.RepeatedScalarFieldContainer[float]
    def __init__(self, x: _Optional[_Iterable[float]] = ..., y: _Optional[_Iterable[float]] = ...) -> None: ...

class YieldTimeSeriesSolarPosition(_message.Message):
    __slots__ = ("ZenithDegrees", "AzimuthDegrees", "ElevationDegrees")
    ZENITHDEGREES_FIELD_NUMBER: _ClassVar[int]
    AZIMUTHDEGREES_FIELD_NUMBER: _ClassVar[int]
    ELEVATIONDEGREES_FIELD_NUMBER: _ClassVar[int]
    ZenithDegrees: float
    AzimuthDegrees: float
    ElevationDegrees: float
    def __init__(self, ZenithDegrees: _Optional[float] = ..., AzimuthDegrees: _Optional[float] = ..., ElevationDegrees: _Optional[float] = ...) -> None: ...

class YieldTimeSeriesWeather(_message.Message):
    __slots__ = ("GHI", "DHI", "DNI", "DiffuseFraction", "AmbientTemp", "WS", "WD", "Cloud")
    GHI_FIELD_NUMBER: _ClassVar[int]
    DHI_FIELD_NUMBER: _ClassVar[int]
    DNI_FIELD_NUMBER: _ClassVar[int]
    DIFFUSEFRACTION_FIELD_NUMBER: _ClassVar[int]
    AMBIENTTEMP_FIELD_NUMBER: _ClassVar[int]
    WS_FIELD_NUMBER: _ClassVar[int]
    WD_FIELD_NUMBER: _ClassVar[int]
    CLOUD_FIELD_NUMBER: _ClassVar[int]
    GHI: float
    DHI: float
    DNI: float
    DiffuseFraction: float
    AmbientTemp: float
    WS: float
    WD: float
    Cloud: float
    def __init__(self, GHI: _Optional[float] = ..., DHI: _Optional[float] = ..., DNI: _Optional[float] = ..., DiffuseFraction: _Optional[float] = ..., AmbientTemp: _Optional[float] = ..., WS: _Optional[float] = ..., WD: _Optional[float] = ..., Cloud: _Optional[float] = ...) -> None: ...



import csv
import math
from datetime import datetime, timezone

from pvl_p90_client.grpcclient import weatherData_pb2
from pvl_p90_client.grpcclient.uncertaintyMessages_pb2 import (
    Distribution,
    DistributionFunction,
    DistributionInput,
    DistributionType,
    ElectricalSettings,
    ModuleInfo,
    OperationalSettings,
    OpticalSettings,
    ResultOptions,
    SystemInfo,
    ThermalSettings,
    TimeStepData,
    TrackingType,
    UncertaintyRequest,
    UncertaintySimulationOptions,
    YieldTimeSeriesSolarPosition,
    YieldTimeSeriesWeather,
)

from .exception import TimeStepDataError


def format_number_for_display(value: float, precision: int = 4) -> str:
    """Format a number for human-readable display.

    Uses scientific notation for very large numbers (>= 1e9) or very small numbers (<= 1e-4),
    otherwise uses fixed-point notation with the specified precision.

    Args:
        value: The number to format
        precision: Number of decimal places for fixed-point notation

    Returns:
        Formatted string representation of the number
    """
    if math.isinf(value):
        return "∞" if value > 0 else "-∞"
    elif math.isnan(value):
        return "NaN"
    elif abs(value) >= 1e9 or (abs(value) <= 1e-4 and value != 0):
        return f"{value:.{precision}e}"
    else:
        return f"{value:.{precision}f}"


def _extract_float_from_nullable(nullable_float: weatherData_pb2.NullableFloat) -> float | None:
    """Extract float value from NullableFloat, returning None if null."""
    if hasattr(nullable_float, "null") and nullable_float.HasField("null"):
        return None
    return nullable_float.data if hasattr(nullable_float, "data") else None


def build_module_info(
    length: float | None = None,
    width: float | None = None,
    cell_to_cell_mismatch: float | None = None,
    power_temperature_coefficient: float | None = None,
    power_at_stc: float | None = None,
    bifaciality: float | None = None,
) -> ModuleInfo:
    """Builds and returns a ModuleInfo message with optional values."""
    module_info = ModuleInfo()
    if length is not None:
        module_info.LengthInM = length
    if width is not None:
        module_info.WidthInM = width
    if cell_to_cell_mismatch is not None:
        module_info.CellToCellMismatch = cell_to_cell_mismatch
    if power_temperature_coefficient is not None:
        module_info.ModulePowerTemperatureCoefficient = power_temperature_coefficient
    if power_at_stc is not None:
        module_info.PowerRatingAtSTCInW = power_at_stc
    if bifaciality is not None:
        module_info.Bifaciality = bifaciality
    return module_info


def build_system_info(
    number_of_inverters: int | None = None,
    strings_per_inverter: int | None = None,
    modules_per_string: int | None = None,
    row_pitch: float | None = None,
    module_azimuth: float | None = None,
    module_tilt: float | None = None,
    module_tracking: TrackingType | None = None,
    module_tilt_limit: float | None = None,
    module_height: float | None = None,
) -> SystemInfo:
    """Builds and returns a SystemInfo message with optional values."""
    system_info = SystemInfo()

    if strings_per_inverter is not None:
        system_info.StringsPerInverter = strings_per_inverter
    if modules_per_string is not None:
        system_info.ModulesPerString = modules_per_string
    if row_pitch is not None:
        system_info.RowPitchInM = row_pitch
    if module_azimuth is not None:
        system_info.ModuleAzimuthInDegrees = module_azimuth
    if number_of_inverters is not None:
        system_info.NumberOfInverters = number_of_inverters
    if module_tilt is not None:
        system_info.FallbackModuleTiltInDegrees = module_tilt
    if module_tracking is not None:
        system_info.ModuleTracking = module_tracking
    if module_tilt_limit is not None:
        system_info.TiltLimitInDegrees = module_tilt_limit
    if module_height is not None:
        system_info.ModuleHeight = module_height

    return system_info


def build_electrical_settings(
    max_power_tracking_loss: float | None = None,
    inverter_efficiency: float | None = None,
    inverter_wiring_loss: float | None = None,
    string_wiring_loss: float | None = None,
    string_to_string_mismatch: float | None = None,
    module_to_module_mismatch: float | None = None,
    inverter_to_inverter_mismatch: float | None = None,
    inverter_clipping: float | None = None,
) -> ElectricalSettings:
    """Builds and returns an ElectricalSettings message with optional values."""
    electrical = ElectricalSettings()
    if inverter_efficiency is not None:
        electrical.InverterEfficiency = inverter_efficiency
    if module_to_module_mismatch is not None:
        electrical.ModuleToModuleMismatch = module_to_module_mismatch
    if string_wiring_loss is not None:
        electrical.StringWiringLoss = string_wiring_loss
    if max_power_tracking_loss is not None:
        electrical.MaxPowerTrackingLoss = max_power_tracking_loss
    if inverter_wiring_loss is not None:
        electrical.InverterWiringLoss = inverter_wiring_loss
    if string_to_string_mismatch is not None:
        electrical.StringToStringMismatch = string_to_string_mismatch
    if inverter_to_inverter_mismatch is not None:
        electrical.InverterToInverterMismatch = inverter_to_inverter_mismatch
    if inverter_clipping is not None:
        electrical.InverterClipping = inverter_clipping

    return electrical


def build_optical_settings(
    beam_multiplier_front: float | None = None,
    beam_multiplier_rear: float | None = None,
    isotropic_multiplier_front: float | None = None,
    isotropic_multiplier_rear: float | None = None,
    albedo: float | None = None,
    soiling_front: float | None = None,
    soiling_rear: float | None = None,
    spectral_correction: float | None = None,
    rear_transmission_factor: float | None = None,
    rear_structural_shading_factor: float | None = None,
) -> OpticalSettings:
    """Builds and returns an OpticalSettings message with optional values."""
    optical = OpticalSettings()

    if beam_multiplier_front is not None:
        optical.BeamMultiplierFront = beam_multiplier_front
    if beam_multiplier_rear is not None:
        optical.BeamMultiplierRear = beam_multiplier_rear
    if isotropic_multiplier_front is not None:
        optical.IsotropicMultiplierFront = isotropic_multiplier_front
    if isotropic_multiplier_rear is not None:
        optical.IsotropicMultiplierRear = isotropic_multiplier_rear
    if albedo is not None:
        optical.FallbackAlbedo = albedo
    if soiling_front is not None:
        optical.FallbackSoilingFront = soiling_front
    if soiling_rear is not None:
        optical.FallbackSoilingRear = soiling_rear
    if spectral_correction is not None:
        optical.FallbackSpectralCorrection = spectral_correction
    if rear_transmission_factor is not None:
        optical.FallbackRearTransmissionFactor = rear_transmission_factor
    if rear_structural_shading_factor is not None:
        optical.FallbackRearStructuralShadingFactor = rear_structural_shading_factor

    return optical


def build_thermal_settings(
    uc: float | None = None,
    uv: float | None = None,
    alpha: float | None = None,
) -> ThermalSettings:
    thermal = ThermalSettings()

    if uc is not None:
        thermal.Uc = uc
    if uv is not None:
        thermal.Uv = uv
    if alpha is not None:
        thermal.Alpha = alpha
    return thermal


def build_operational_settings(
    annual_degradation_rate: float | None = None,
    curtailment: float | None = None,
    availability: float | None = None,
    dc_health: float | None = None,
    undulating_ground: bool | None = None,
    yield_modifier: float | None = None,
) -> OperationalSettings:
    operational = OperationalSettings()

    if annual_degradation_rate is not None:
        operational.AnnualDegradationRate = annual_degradation_rate
    if curtailment is not None:
        operational.Curtailment = curtailment
    if availability is not None:
        operational.Availability = availability
    if dc_health is not None:
        operational.DCHealth = dc_health
    if undulating_ground is not None:
        operational.UndulatingGround = undulating_ground
    if yield_modifier is not None:
        operational.YieldModifier = yield_modifier

    return operational


def build_request(
    time_step_data: list[TimeStepData],
    module: ModuleInfo | None = None,
    system: SystemInfo | None = None,
    electrical: ElectricalSettings | None = None,
    optical: OpticalSettings | None = None,
    thermal: ThermalSettings | None = None,
    operational: OperationalSettings | None = None,
    distributions: list[Distribution] | None = None,
    simulation_options: UncertaintySimulationOptions | None = None,
    result_options: ResultOptions | None = None,
) -> UncertaintyRequest:
    request = UncertaintyRequest()

    if module is not None:
        request.Module.CopyFrom(module)
    if system is not None:
        request.System.CopyFrom(system)
    if electrical is not None:
        request.Electrical.CopyFrom(electrical)
    if optical is not None:
        request.Optical.CopyFrom(optical)
    if thermal is not None:
        request.Thermal.CopyFrom(thermal)
    if operational is not None:
        request.Operation.CopyFrom(operational)
    if simulation_options is not None:
        request.SimulationOptions.CopyFrom(simulation_options)
    if result_options is not None:
        request.ResultOptions.CopyFrom(result_options)
    else:
        # DefaulResult options are required by the service and should be set if not provided
        request.ResultOptions.CopyFrom(build_result_options())

    # Add modifiers if provided
    if distributions is not None:
        request.Distributions.extend(distributions)

    # Add Typical Mean Year data
    request.TMYDataSet.TimeStepDataPoints.extend(time_step_data)
    # Add any default settings if needed
    return request


def build_result_options(bin_min: float = 0.75, bin_delta: float = 0.01, p_values: list[int] | None = None) -> ResultOptions:
    """Builds and returns a ResultOptions message with default values."""
    if p_values is None:
        p_values = [5, 10, 50, 90, 95]
    result_options = ResultOptions()
    result_options.BinDelta = bin_delta
    result_options.PValues.extend(p_values)
    result_options.BinMin = bin_min
    return result_options


def build_simulation_options(number_of_years: int = 5, number_of_simulations: int = 100) -> UncertaintySimulationOptions:
    """Builds and returns a SimulationOptions message with default values."""
    simulation_options = UncertaintySimulationOptions()
    simulation_options.NumberOfYears = number_of_years
    simulation_options.NumberOfSimulations = number_of_simulations
    return simulation_options


def build_distribution(
    input: DistributionInput,
    sim_to_sim_distribution: DistributionFunction | None = None,
    yr_to_yr_distribution: DistributionFunction | None = None,
    step_to_step_distribution: DistributionFunction | None = None,
) -> Distribution:  # type: ignore
    """Builds and returns a Modifier message."""
    distribution = Distribution()
    distribution.Input = input

    if sim_to_sim_distribution is not None:
        distribution.SimToSimDistribution.CopyFrom(sim_to_sim_distribution)
    if yr_to_yr_distribution is not None:
        distribution.YrToYrDistribution.CopyFrom(yr_to_yr_distribution)
    if step_to_step_distribution is not None:
        distribution.StepToStepDistribution.CopyFrom(step_to_step_distribution)
    return distribution


def build_gaussian_distribution(
    x0: float,
    sigma: float,
    fmax: float | None = None,
    num_points_in_probability_function: int | None = None,
    lower_limit: float | None = None,
    upper_limit: float | None = None,
) -> DistributionFunction:
    """Builds and returns an ErrorFunction message."""
    distribution = DistributionFunction()
    distribution.Type = DistributionType.Gaussian
    distribution.Gaussian.x0 = x0
    distribution.Gaussian.sigma = sigma

    if fmax is not None:
        distribution.Gaussian.fmax = fmax
    if num_points_in_probability_function is not None:
        distribution.Gaussian.numPointsInProbabilityFunction = num_points_in_probability_function
    if lower_limit is not None:
        distribution.Gaussian.xLowerLimit = lower_limit
    if upper_limit is not None:
        distribution.Gaussian.xUpperLimit = upper_limit

    return distribution


def build_skewed_gaussian_distribution(
    alpha: float,
    zeta: float,
    omega: float,
    fmax: float | None = None,
    num_points_in_probability_function: int | None = None,
) -> DistributionFunction:
    """Builds and returns an ErrorFunction message."""
    distribution = DistributionFunction()
    distribution.Type = DistributionType.SkewedGaussian
    distribution.SkewedGaussian.alpha = alpha
    distribution.SkewedGaussian.zeta = zeta
    distribution.SkewedGaussian.omega = omega

    if fmax is not None:
        distribution.SkewedGaussian.fmax = fmax
    if num_points_in_probability_function is not None:
        distribution.SkewedGaussian.numPointsInProbabilityFunction = num_points_in_probability_function

    return distribution


def build_weibull_distribution(x0: float, lambda_term: float, k: float, has_positive_polarity: bool | None = None) -> DistributionFunction:
    """Builds and returns an ErrorFunction message."""
    distribution = DistributionFunction()
    distribution.Type = DistributionType.Weibull
    distribution.Weibull.x0 = x0
    distribution.Weibull.k = k
    # Note: 'lambda' is a Python keyword, so we use getattr/setattr
    setattr(distribution.Weibull, "lambda", lambda_term)

    if has_positive_polarity is not None:
        distribution.Weibull.hasPositivePolarity = has_positive_polarity

    return distribution


def build_arbitrary_distribution(x: list[float], y: list[float]) -> DistributionFunction:
    distribution = DistributionFunction()
    distribution.Type = DistributionType.Arbitrary
    # Create the nested message properly
    arbitrary_func = distribution.Arbitrary
    arbitrary_func.x[:] = x  # Use slice assignment for repeated fields
    arbitrary_func.y[:] = y  # Use slice assignment for repeated fields
    return distribution


def load_weather_data_from_pvw_file(file_path: str) -> list[TimeStepData] | None:
    """Load weather data from a .pvw file and convert it to WeatherData messages.

    Returns None if the file cannot be loaded or parsed.
    """
    try:
        weather_data = []
        with open(file_path, "rb") as f:
            data = f.read()
            weather_setting = weatherData_pb2.WeatherSetting()
            weather_setting.ParseFromString(data)
            for row in weather_setting.weatherRows:
                # Convert temperature from Celsius to Kelvin if available
                temp_celsius = _extract_float_from_nullable(row.DryBulbTemperature_DegreesCeliusAt2Metres)
                temp_kelvin = celsius_to_kelvin(temp_celsius) if temp_celsius is not None else None

                point = TimeStepData(
                    EndOfPeriodUTC=row.ObservationTime_UTC,
                    SolarPos=YieldTimeSeriesSolarPosition(
                        AzimuthDegrees=_extract_float_from_nullable(row.SolarAzimuth), ZenithDegrees=_extract_float_from_nullable(row.SolarZenith)
                    ),
                    Weather=YieldTimeSeriesWeather(
                        AmbientTemp=temp_kelvin,
                        DHI=_extract_float_from_nullable(row.DiffuseHorizontalIrradiance_WattsPerMetreSquared),
                        GHI=_extract_float_from_nullable(row.GlobalHorizontalIrradiance_WattsPerMetreSquared),
                        DNI=_extract_float_from_nullable(row.DirectNormalIrradiance_WattsPerMetreSquared),
                        WS=_extract_float_from_nullable(row.WindVelocity_MetresPerSecondAt10Metres),
                    ),
                )

                weather_data.append(point)
        return weather_data
    except (FileNotFoundError, OSError) as e:
        print(f"Error loading weather data from {file_path}: {e}")
        raise TimeStepDataError("Failed to load weather data") from e
    except Exception as e:
        print(f"Error parsing weather data from {file_path}: {e}")
        raise TimeStepDataError("Failed to load weather data") from e


def load_time_step_data_from_csv(file_path: str) -> list[TimeStepData]:
    """Load weather data from a CSV file and convert it to TimeStepData messages.

    The CSV file must start with a version header: # P90_CSV_VERSION=1.0

    The CSV file is expected to have the following columns:
    - EndOfPeriodUTC (dd/mm/yyyy HH:mm)
    - SolarAzimuth (degrees)
    - SolarZenith (degrees)
    - AmbientTemp (celsius)
    - DHI (W/m2)
    - GHI (W/m2)
    - DNI (W/m2)
    - WS (m/s)
    - Tilt (degrees) - column required but values may be blank
    - Albedo - column required but values may be blank
    - SoilingFrontRate - column required but values may be blank
    - SoilingRearRate - column required but values may be blank
    - RearTransmissionFactor (%) - column required but values may be blank, percentage format
    - RearStructuralShadingFactor (%) - column required but values may be blank, percentage format

    Returns a list of TimeStepData messages.

    Raises:
        TimeStepDataError: If the file format is invalid, version is missing/unsupported, or required columns are missing.
    """
    # Define expected columns (all must be present, but values may be blank)
    expected_columns = {
        "EndOfPeriodUTC (dd/mm/yyyy HH:mm)",
        "SolarAzimuth (degrees)",
        "SolarZenith (degrees)",
        "AmbientTemp (celsius)",
        "DHI (W/m2)",
        "GHI (W/m2)",
        "DNI (W/m2)",
        "WS (m/s)",
        "Tilt (degrees)",
        "Albedo",
        "SoilingFrontRate",
        "SoilingRearRate",
        "RearTransmissionFactor (%)",
        "RearStructuralShadingFactor (%)",
    }

    weather_data = []
    try:
        with open(file_path, newline="") as csvfile:
            # Check for required version header
            first_line = csvfile.readline().strip()

            if not first_line.startswith("# P90_CSV_VERSION="):
                raise TimeStepDataError(f"CSV file {file_path} is missing required version header. Expected: # P90_CSV_VERSION=1.0")

            # Extract and validate version
            version = first_line.split("=")[1].strip()
            if version != "1.0":
                raise TimeStepDataError(f"Unsupported CSV format version: {version}. Only version 1.0 is currently supported.")

            # Save position after version header
            header_position = csvfile.tell()

            # First, try to detect if it's a valid CSV with comma separator
            sample = csvfile.read(1024)

            # Make sure sample ends with a complete line
            if sample and not sample.endswith("\n"):
                last_newline = sample.rfind("\n")
                if last_newline != -1:
                    sample = sample[: last_newline + 1]

            # Use csv.Sniffer to detect format
            sniffer = csv.Sniffer()
            try:
                dialect = sniffer.sniff(sample, delimiters=",")
                if dialect.delimiter != ",":
                    raise TimeStepDataError(f"File {file_path} must use comma as separator, found '{dialect.delimiter}'")
            except csv.Error as e:
                raise TimeStepDataError(f"File {file_path} does not appear to be a valid CSV format") from e

            # Check if file has a header
            if not sniffer.has_header(sample):
                raise TimeStepDataError(f"File {file_path} must have a header row")

            # Seek back to position after version header to read the CSV
            csvfile.seek(header_position)

            # Read the CSV with detected dialect
            reader = csv.DictReader(csvfile, dialect=dialect)

            # Validate that all expected columns are present
            if reader.fieldnames is None:
                raise TimeStepDataError("Unable to read CSV header")

            actual_columns = set(reader.fieldnames)
            missing_columns = expected_columns - actual_columns

            if missing_columns:
                missing_list = sorted(missing_columns)
                raise TimeStepDataError(f"Missing required columns: {', '.join(missing_list)}")

            # Process the data rows
            for row in reader:
                # Parse the datetime string (expected format: dd/mm/yyyy HH:mm)
                try:
                    end_time = datetime.strptime(row["EndOfPeriodUTC (dd/mm/yyyy HH:mm)"], "%d/%m/%Y %H:%M")
                except ValueError as e:
                    date_value = row["EndOfPeriodUTC (dd/mm/yyyy HH:mm)"]
                    raise TimeStepDataError(f"Invalid date format in EndOfPeriodUTC: {date_value}. Expected format: dd/mm/yyyy HH:mm") from e

                temp_celsius = float(row["AmbientTemp (celsius)"]) if row["AmbientTemp (celsius)"] else None
                temp_kelvin = celsius_to_kelvin(temp_celsius) if temp_celsius is not None else None

                # Create the TimeStepData object with all fields
                point = TimeStepData(
                    EndOfPeriodUTC=end_time,
                    SolarPos=YieldTimeSeriesSolarPosition(
                        AzimuthDegrees=float(row["SolarAzimuth (degrees)"]) if row["SolarAzimuth (degrees)"] else None,
                        ZenithDegrees=float(row["SolarZenith (degrees)"]) if row["SolarZenith (degrees)"] else None,
                    ),
                    Weather=YieldTimeSeriesWeather(
                        AmbientTemp=temp_kelvin,
                        DHI=float(row["DHI (W/m2)"]) if row["DHI (W/m2)"] else None,
                        GHI=float(row["GHI (W/m2)"]) if row["GHI (W/m2)"] else None,
                        DNI=float(row["DNI (W/m2)"]) if row["DNI (W/m2)"] else None,
                        WS=float(row["WS (m/s)"]) if row["WS (m/s)"] else None,
                    ),
                )

                # Set optional fields only if they have values
                if row["Tilt (degrees)"].strip():
                    point.TiltAngleInDegrees = float(row["Tilt (degrees)"])

                if row["Albedo"].strip():
                    point.Albedo = float(row["Albedo"])

                if row["SoilingFrontRate"].strip():
                    point.SoilingFrontRate = float(row["SoilingFrontRate"])

                if row["SoilingRearRate"].strip():
                    point.SoilingRearRate = float(row["SoilingRearRate"])

                if row["RearTransmissionFactor (%)"].strip():
                    point.RearTransmissionFactor = float(row["RearTransmissionFactor (%)"]) / 100.0

                if row["RearStructuralShadingFactor (%)"].strip():
                    point.RearStructuralShadingFactor = float(row["RearStructuralShadingFactor (%)"]) / 100.0

                weather_data.append(point)
        return weather_data
    except (FileNotFoundError, OSError) as e:
        print(f"Error loading weather data from {file_path}: {e}")
        raise TimeStepDataError("Failed to load weather data") from e
    except TimeStepDataError:
        # Re-raise WeatherDataError as-is
        raise
    except Exception as e:
        print(f"Error parsing weather data from {file_path}: {e}")
        raise TimeStepDataError("Failed to load weather data") from e


def write_timestep_data_to_csv(file_path: str, time_step_data: list[TimeStepData]) -> None:
    """Save time step data to a CSV file with P90 CSV format versioning."""
    with open(file_path, mode="w", newline="") as csvfile:
        # Write version header first
        csvfile.write("# P90_CSV_VERSION=1.0\n")

        fieldnames = [
            "EndOfPeriodUTC (dd/mm/yyyy HH:mm)",
            "SolarAzimuth (degrees)",
            "SolarZenith (degrees)",
            "AmbientTemp (celsius)",
            "DHI (W/m2)",
            "GHI (W/m2)",
            "DNI (W/m2)",
            "WS (m/s)",
            "Tilt (degrees)",
            "Albedo",
            "SoilingFrontRate",
            "SoilingRearRate",
            "RearTransmissionFactor (%)",
            "RearStructuralShadingFactor (%)",
        ]
        writer = csv.DictWriter(csvfile, fieldnames=fieldnames)
        writer.writeheader()
        for point in time_step_data:
            # Convert protobuf Timestamp to Python datetime if it's a Timestamp object
            end_time = point.EndOfPeriodUTC.ToDatetime(tzinfo=timezone.utc)  # noqa

            writer.writerow(
                {
                    "EndOfPeriodUTC (dd/mm/yyyy HH:mm)": end_time.strftime("%d/%m/%Y %H:%M") if isinstance(end_time, datetime) else str(end_time),
                    "SolarAzimuth (degrees)": point.SolarPos.AzimuthDegrees,
                    "SolarZenith (degrees)": point.SolarPos.ZenithDegrees,
                    "AmbientTemp (celsius)": kelvin_to_celsius(point.Weather.AmbientTemp),
                    "DHI (W/m2)": point.Weather.DHI,
                    "GHI (W/m2)": point.Weather.GHI,
                    "DNI (W/m2)": point.Weather.DNI,
                    "WS (m/s)": point.Weather.WS,
                    "Tilt (degrees)": point.TiltAngleInDegrees if hasattr(point, "TiltAngleInDegrees") and point.TiltAngleInDegrees is not None else "",
                    "Albedo": point.Albedo if hasattr(point, "Albedo") and point.Albedo is not None else "",
                    "SoilingFrontRate": point.SoilingFrontRate if hasattr(point, "SoilingFrontRate") and point.SoilingFrontRate is not None else "",
                    "SoilingRearRate": point.SoilingRearRate if hasattr(point, "SoilingRearRate") and point.SoilingRearRate is not None else "",
                    "RearTransmissionFactor (%)": (
                        point.RearTransmissionFactor * 100.0 if hasattr(point, "RearTransmissionFactor") and point.RearTransmissionFactor is not None else ""
                    ),
                    "RearStructuralShadingFactor (%)": (
                        point.RearStructuralShadingFactor * 100.0
                        if hasattr(point, "RearStructuralShadingFactor") and point.RearStructuralShadingFactor is not None
                        else ""
                    ),
                }
            )


def celsius_to_kelvin(celsius: float | None) -> float | None:
    """Convert Celsius to Kelvin, handling None values."""
    return celsius + 273.15 if celsius is not None else None


def kelvin_to_celsius(kelvin: float | None) -> float | None:
    """Convert Kelvin to Celsius, handling None values."""
    return kelvin - 273.15 if kelvin is not None else None



class P90ClientError(Exception):
    """Base exception for P90Client errors."""


class AuthenticationError(P90ClientError):
    """Raised when authentication fails."""


class P90ConnectionError(P90ClientError):
    """Raised when connection to the service fails."""


class ServiceError(P90ClientError):
    """Raised when the service returns an error."""


class TimeStepDataError(P90ClientError):
    """Raised when there is an error with time step data processing."""



import getpass
import os
import time
from multiprocessing import AuthenticationError

import grpc
import jwt
import requests

from .exception import P90ConnectionError


def login(username=None, password=None) -> grpc.CallCredentials:
    """Login to PV Lighthouse and return call credentials.

    Raises AuthenticationError or P90ConnectionError if authentication fails.
    """
    try:
        # Check for existing valid token
        if os.path.exists(".pvlToken"):
            with open(".pvlToken", encoding="utf-8") as token_file:
                token = token_file.read()
            decoded = jwt.decode(token, options={"verify_signature": False})
            exp_time = decoded.get("exp")
            if exp_time is not None and int(time.time()) < int(exp_time):
                return grpc.access_token_call_credentials(token)

        # Get user credentials
        if not username:
            username = input("Enter username:")

        if not password:
            password = getpass.getpass("Enter password:")

        # Request new token
        post_data = {"grant_type": "password", "username": username, "password": password}
        r = requests.post("https://www.pvlighthouse.com.au/Token", data=post_data, timeout=30)
        r.raise_for_status()
        jwt_obj = r.json()
        new_token = jwt_obj["access_token"]

        # Save token for future use
        with open(".pvlToken", "w", encoding="utf-8") as f:
            f.write(new_token)
        return grpc.access_token_call_credentials(new_token)

    except (requests.RequestException, requests.HTTPError) as e:
        print(f"Authentication failed - network error: {e}")
        raise AuthenticationError("Failed to connect to PV Lighthouse service") from e
    except (jwt.InvalidTokenError, KeyError) as e:
        print(f"Authentication failed - token error: {e}")
        raise AuthenticationError("Failed to connect to PV Lighthouse service") from e
    except OSError as e:
        print(f"Authentication failed - file error: {e}")
        raise P90ConnectionError("Failed to connect to PV Lighthouse service") from e
    except Exception as e:
        print(f"Authentication failed - unexpected error: {e}")
        raise P90ConnectionError("Failed to connect to PV Lighthouse service") from e



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: uncertaintyMessages.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'uncertaintyMessages.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x19uncertaintyMessages.proto\x12\x11grpc.SunSolve.API\x1a\x1fgoogle/protobuf/timestamp.proto\"\xfa\x01\n\x13UncertaintyResponse\x12\x13\n\x0bpctComplete\x18\x01 \x01(\x02\x12J\n\x10SimulationResult\x18\x02 \x01(\x0b\x32..grpc.SunSolve.API.UncertaintySimulationResultH\x00\x12\x38\n\x07Summary\x18\x03 \x01(\x0b\x32%.grpc.SunSolve.API.UncertaintySummaryH\x00\x12>\n\nInputsUsed\x18\x04 \x01(\x0b\x32(.grpc.SunSolve.API.UncertaintyUsedInputsH\x00\x42\x08\n\x06Result\"\xab\x01\n\x1bUncertaintySimulationResult\x12\x17\n\x0fSimulationIndex\x18\x01 \x01(\x02\x12\x12\n\nYearNumber\x18\x02 \x01(\x02\x12\x14\n\x0cTotalDCYield\x18\x03 \x01(\x01\x12\x14\n\x0cTotalACYield\x18\x04 \x01(\x01\x12\x1e\n\x16YearlyAvailableYieldAC\x18\x05 \x01(\x01\x12\x13\n\x0b\x44\x65gradation\x18\x06 \x01(\x01\"\xb9\x01\n\x12UncertaintySummary\x12/\n\x0cYearlyPValue\x18\x01 \x03(\x0b\x32\x19.grpc.SunSolve.API.PValue\x12\x35\n\x0fYearlyHistogram\x18\x02 \x03(\x0b\x32\x1c.grpc.SunSolve.API.Histogram\x12;\n\x12YearlyP50Deviation\x18\x03 \x03(\x0b\x32\x1f.grpc.SunSolve.API.P50Deviation\"\xda\x03\n\x15UncertaintyUsedInputs\x12-\n\x06Module\x18\x01 \x01(\x0b\x32\x1d.grpc.SunSolve.API.ModuleInfo\x12-\n\x06System\x18\x02 \x01(\x0b\x32\x1d.grpc.SunSolve.API.SystemInfo\x12\x39\n\nElectrical\x18\x03 \x01(\x0b\x32%.grpc.SunSolve.API.ElectricalSettings\x12\x33\n\x07Optical\x18\x04 \x01(\x0b\x32\".grpc.SunSolve.API.OpticalSettings\x12\x33\n\x07Thermal\x18\x05 \x01(\x0b\x32\".grpc.SunSolve.API.ThermalSettings\x12\x39\n\tOperation\x18\x06 \x01(\x0b\x32&.grpc.SunSolve.API.OperationalSettings\x12J\n\x11SimulationOptions\x18\x07 \x01(\x0b\x32/.grpc.SunSolve.API.UncertaintySimulationOptions\x12\x37\n\rResultOptions\x18\x08 \x01(\x0b\x32 .grpc.SunSolve.API.ResultOptions\"u\n\x06PValue\x12\x0c\n\x04Year\x18\x01 \x01(\x05\x12\t\n\x01P\x18\x02 \x01(\x05\x12\x12\n\x05Value\x18\x03 \x01(\x01H\x00\x88\x01\x01\x12\x14\n\x0cP50Deviation\x18\x04 \x01(\x01\x12\x13\n\x06\x43onf95\x18\x05 \x01(\x01H\x01\x88\x01\x01\x42\x08\n\x06_ValueB\t\n\x07_Conf95\"\'\n\tHistogram\x12\x0c\n\x04Year\x18\x01 \x01(\x05\x12\x0c\n\x04\x62ins\x18\x02 \x03(\x05\"+\n\x0cP50Deviation\x12\x0c\n\x04Year\x18\x01 \x01(\x05\x12\r\n\x05Value\x18\x02 \x01(\x01\"\xc3\x06\n\x12UncertaintyRequest\x12;\n\nTMYDataSet\x18\x01 \x01(\x0b\x32%.grpc.SunSolve.API.TMYTimeStepDataSetH\x00\x12G\n\x13\x41\x63tualYearlyDataSet\x18\x02 \x01(\x0b\x32(.grpc.SunSolve.API.ActualTimeStepDataSetH\x00\x12\x32\n\x06Module\x18\x03 \x01(\x0b\x32\x1d.grpc.SunSolve.API.ModuleInfoH\x01\x88\x01\x01\x12\x32\n\x06System\x18\x04 \x01(\x0b\x32\x1d.grpc.SunSolve.API.SystemInfoH\x02\x88\x01\x01\x12>\n\nElectrical\x18\x05 \x01(\x0b\x32%.grpc.SunSolve.API.ElectricalSettingsH\x03\x88\x01\x01\x12\x38\n\x07Optical\x18\x06 \x01(\x0b\x32\".grpc.SunSolve.API.OpticalSettingsH\x04\x88\x01\x01\x12\x38\n\x07Thermal\x18\x07 \x01(\x0b\x32\".grpc.SunSolve.API.ThermalSettingsH\x05\x88\x01\x01\x12>\n\tOperation\x18\x08 \x01(\x0b\x32&.grpc.SunSolve.API.OperationalSettingsH\x06\x88\x01\x01\x12O\n\x11SimulationOptions\x18\t \x01(\x0b\x32/.grpc.SunSolve.API.UncertaintySimulationOptionsH\x07\x88\x01\x01\x12<\n\rResultOptions\x18\n \x01(\x0b\x32 .grpc.SunSolve.API.ResultOptionsH\x08\x88\x01\x01\x12\x36\n\rDistributions\x18\x0b \x03(\x0b\x32\x1f.grpc.SunSolve.API.DistributionB\x11\n\x0fTimeStepDataSetB\t\n\x07_ModuleB\t\n\x07_SystemB\r\n\x0b_ElectricalB\n\n\x08_OpticalB\n\n\x08_ThermalB\x0c\n\n_OperationB\x14\n\x12_SimulationOptionsB\x10\n\x0e_ResultOptions\"Q\n\x12TMYTimeStepDataSet\x12;\n\x12TimeStepDataPoints\x18\x01 \x03(\x0b\x32\x1f.grpc.SunSolve.API.TimeStepData\"a\n\x15\x41\x63tualTimeStepDataSet\x12H\n\x12TimeStepDataPoints\x18\x01 \x03(\x0b\x32,.grpc.SunSolve.API.ActualTimeStepDataForYear\"k\n\x19\x41\x63tualTimeStepDataForYear\x12\x11\n\tYearIndex\x18\x01 \x01(\x05\x12;\n\x12TimeStepDataPoints\x18\x02 \x03(\x0b\x32\x1f.grpc.SunSolve.API.TimeStepData\"\x89\x04\n\x0cTimeStepData\x12\x32\n\x0e\x45ndOfPeriodUTC\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x41\n\x08SolarPos\x18\x02 \x01(\x0b\x32/.grpc.SunSolve.API.YieldTimeSeriesSolarPosition\x12:\n\x07Weather\x18\x03 \x01(\x0b\x32).grpc.SunSolve.API.YieldTimeSeriesWeather\x12\x1f\n\x12TiltAngleInDegrees\x18\x04 \x01(\x01H\x00\x88\x01\x01\x12\x13\n\x06\x41lbedo\x18\x05 \x01(\x01H\x01\x88\x01\x01\x12\x1d\n\x10SoilingFrontRate\x18\x06 \x01(\x01H\x02\x88\x01\x01\x12\x1c\n\x0fSoilingRearRate\x18\x07 \x01(\x01H\x03\x88\x01\x01\x12#\n\x16RearTransmissionFactor\x18\x08 \x01(\x01H\x04\x88\x01\x01\x12(\n\x1bRearStructuralShadingFactor\x18\t \x01(\x01H\x05\x88\x01\x01\x42\x15\n\x13_TiltAngleInDegreesB\t\n\x07_AlbedoB\x13\n\x11_SoilingFrontRateB\x12\n\x10_SoilingRearRateB\x19\n\x17_RearTransmissionFactorB\x1e\n\x1c_RearStructuralShadingFactor\"\xc8\x02\n\nModuleInfo\x12\x16\n\tLengthInM\x18\x01 \x01(\x01H\x00\x88\x01\x01\x12\x15\n\x08WidthInM\x18\x02 \x01(\x01H\x01\x88\x01\x01\x12\x1f\n\x12\x43\x65llToCellMismatch\x18\x03 \x01(\x01H\x02\x88\x01\x01\x12.\n!ModulePowerTemperatureCoefficient\x18\x04 \x01(\x01H\x03\x88\x01\x01\x12 \n\x13PowerRatingAtSTCInW\x18\x05 \x01(\x01H\x04\x88\x01\x01\x12\x18\n\x0b\x42ifaciality\x18\x06 \x01(\x01H\x05\x88\x01\x01\x42\x0c\n\n_LengthInMB\x0b\n\t_WidthInMB\x15\n\x13_CellToCellMismatchB$\n\"_ModulePowerTemperatureCoefficientB\x16\n\x14_PowerRatingAtSTCInWB\x0e\n\x0c_Bifaciality\"\x97\x04\n\nSystemInfo\x12\x1e\n\x11NumberOfInverters\x18\x01 \x01(\x05H\x00\x88\x01\x01\x12\x1f\n\x12StringsPerInverter\x18\x02 \x01(\x05H\x01\x88\x01\x01\x12\x1d\n\x10ModulesPerString\x18\x03 \x01(\x05H\x02\x88\x01\x01\x12\x18\n\x0bRowPitchInM\x18\x04 \x01(\x01H\x03\x88\x01\x01\x12#\n\x16ModuleAzimuthInDegrees\x18\x05 \x01(\x01H\x04\x88\x01\x01\x12(\n\x1b\x46\x61llbackModuleTiltInDegrees\x18\x06 \x01(\x01H\x05\x88\x01\x01\x12<\n\x0eModuleTracking\x18\x07 \x01(\x0e\x32\x1f.grpc.SunSolve.API.TrackingTypeH\x06\x88\x01\x01\x12\x1f\n\x12TiltLimitInDegrees\x18\x08 \x01(\x01H\x07\x88\x01\x01\x12\x19\n\x0cModuleHeight\x18\t \x01(\x01H\x08\x88\x01\x01\x42\x14\n\x12_NumberOfInvertersB\x15\n\x13_StringsPerInverterB\x13\n\x11_ModulesPerStringB\x0e\n\x0c_RowPitchInMB\x19\n\x17_ModuleAzimuthInDegreesB\x1e\n\x1c_FallbackModuleTiltInDegreesB\x11\n\x0f_ModuleTrackingB\x15\n\x13_TiltLimitInDegreesB\x0f\n\r_ModuleHeight\"\xd3\x05\n\x0fOpticalSettings\x12 \n\x13\x42\x65\x61mMultiplierFront\x18\x01 \x01(\x01H\x00\x88\x01\x01\x12%\n\x18IsotropicMultiplierFront\x18\x02 \x01(\x01H\x01\x88\x01\x01\x12\x1f\n\x12\x42\x65\x61mMultiplierRear\x18\x03 \x01(\x01H\x02\x88\x01\x01\x12$\n\x17IsotropicMultiplierRear\x18\x04 \x01(\x01H\x03\x88\x01\x01\x12\x1b\n\x0e\x46\x61llbackAlbedo\x18\x05 \x01(\x01H\x04\x88\x01\x01\x12!\n\x14\x46\x61llbackSoilingFront\x18\x06 \x01(\x01H\x05\x88\x01\x01\x12 \n\x13\x46\x61llbackSoilingRear\x18\x07 \x01(\x01H\x06\x88\x01\x01\x12\'\n\x1a\x46\x61llbackSpectralCorrection\x18\x08 \x01(\x01H\x07\x88\x01\x01\x12\x1c\n\x0f\x45xtraIrradiance\x18\t \x01(\x01H\x08\x88\x01\x01\x12+\n\x1e\x46\x61llbackRearTransmissionFactor\x18\n \x01(\x01H\t\x88\x01\x01\x12\x30\n#FallbackRearStructuralShadingFactor\x18\x0b \x01(\x01H\n\x88\x01\x01\x42\x16\n\x14_BeamMultiplierFrontB\x1b\n\x19_IsotropicMultiplierFrontB\x15\n\x13_BeamMultiplierRearB\x1a\n\x18_IsotropicMultiplierRearB\x11\n\x0f_FallbackAlbedoB\x17\n\x15_FallbackSoilingFrontB\x16\n\x14_FallbackSoilingRearB\x1d\n\x1b_FallbackSpectralCorrectionB\x12\n\x10_ExtraIrradianceB!\n\x1f_FallbackRearTransmissionFactorB&\n$_FallbackRearStructuralShadingFactor\"\xf0\x03\n\x12\x45lectricalSettings\x12\x1f\n\x12InverterEfficiency\x18\x01 \x01(\x01H\x00\x88\x01\x01\x12\'\n\x1aInverterToInverterMismatch\x18\x02 \x01(\x01H\x01\x88\x01\x01\x12#\n\x16StringToStringMismatch\x18\x03 \x01(\x01H\x02\x88\x01\x01\x12#\n\x16ModuleToModuleMismatch\x18\x04 \x01(\x01H\x03\x88\x01\x01\x12\x1f\n\x12InverterWiringLoss\x18\x05 \x01(\x01H\x04\x88\x01\x01\x12\x1d\n\x10StringWiringLoss\x18\x06 \x01(\x01H\x05\x88\x01\x01\x12!\n\x14MaxPowerTrackingLoss\x18\x07 \x01(\x01H\x06\x88\x01\x01\x12\x1d\n\x10InverterClipping\x18\x08 \x01(\x01H\x07\x88\x01\x01\x42\x15\n\x13_InverterEfficiencyB\x1d\n\x1b_InverterToInverterMismatchB\x19\n\x17_StringToStringMismatchB\x19\n\x17_ModuleToModuleMismatchB\x15\n\x13_InverterWiringLossB\x13\n\x11_StringWiringLossB\x17\n\x15_MaxPowerTrackingLossB\x13\n\x11_InverterClipping\"_\n\x0fThermalSettings\x12\x0f\n\x02Uc\x18\x01 \x01(\x01H\x00\x88\x01\x01\x12\x0f\n\x02Uv\x18\x02 \x01(\x01H\x01\x88\x01\x01\x12\x12\n\x05\x41lpha\x18\x03 \x01(\x01H\x02\x88\x01\x01\x42\x05\n\x03_UcB\x05\n\x03_UvB\x08\n\x06_Alpha\"\xaf\x02\n\x13OperationalSettings\x12\"\n\x15\x41nnualDegradationRate\x18\x01 \x01(\x01H\x00\x88\x01\x01\x12\x15\n\x08\x44\x43Health\x18\x02 \x01(\x01H\x01\x88\x01\x01\x12\x19\n\x0c\x41vailability\x18\x03 \x01(\x01H\x02\x88\x01\x01\x12\x18\n\x0b\x43urtailment\x18\x04 \x01(\x01H\x03\x88\x01\x01\x12\x1a\n\rYieldModifier\x18\x05 \x01(\x01H\x04\x88\x01\x01\x12\x1d\n\x10UndulatingGround\x18\x06 \x01(\x01H\x05\x88\x01\x01\x42\x18\n\x16_AnnualDegradationRateB\x0b\n\t_DCHealthB\x0f\n\r_AvailabilityB\x0e\n\x0c_CurtailmentB\x10\n\x0e_YieldModifierB\x13\n\x11_UndulatingGround\"\x86\x01\n\x1cUncertaintySimulationOptions\x12 \n\x13NumberOfSimulations\x18\x01 \x01(\x05H\x00\x88\x01\x01\x12\x1a\n\rNumberOfYears\x18\x02 \x01(\x05H\x01\x88\x01\x01\x42\x16\n\x14_NumberOfSimulationsB\x10\n\x0e_NumberOfYears\"\x9c\x01\n\rResultOptions\x12\x13\n\x06\x42inMin\x18\x01 \x01(\x01H\x00\x88\x01\x01\x12\x15\n\x08\x42inDelta\x18\x02 \x01(\x01H\x01\x88\x01\x01\x12\x0f\n\x07PValues\x18\x03 \x03(\x05\x12\x1f\n\x12IncludeYieldResult\x18\x04 \x01(\x08H\x02\x88\x01\x01\x42\t\n\x07_BinMinB\x0b\n\t_BinDeltaB\x15\n\x13_IncludeYieldResult\"\xf2\x02\n\x0c\x44istribution\x12\x33\n\x05Input\x18\x01 \x01(\x0e\x32$.grpc.SunSolve.API.DistributionInput\x12J\n\x14SimToSimDistribution\x18\x02 \x01(\x0b\x32\'.grpc.SunSolve.API.DistributionFunctionH\x00\x88\x01\x01\x12H\n\x12YrToYrDistribution\x18\x03 \x01(\x0b\x32\'.grpc.SunSolve.API.DistributionFunctionH\x01\x88\x01\x01\x12L\n\x16StepToStepDistribution\x18\x04 \x01(\x0b\x32\'.grpc.SunSolve.API.DistributionFunctionH\x02\x88\x01\x01\x42\x17\n\x15_SimToSimDistributionB\x15\n\x13_YrToYrDistributionB\x19\n\x17_StepToStepDistribution\"\x92\x03\n\x14\x44istributionFunction\x12\x31\n\x04Type\x18\x01 \x01(\x0e\x32#.grpc.SunSolve.API.DistributionType\x12;\n\x08\x43onstant\x18\x02 \x01(\x0b\x32\'.grpc.SunSolve.API.ConstantDistributionH\x00\x12;\n\x08Gaussian\x18\x03 \x01(\x0b\x32\'.grpc.SunSolve.API.GaussianDistributionH\x00\x12\x39\n\x07Weibull\x18\x04 \x01(\x0b\x32&.grpc.SunSolve.API.WeibullDistributionH\x00\x12G\n\x0eSkewedGaussian\x18\x05 \x01(\x0b\x32-.grpc.SunSolve.API.SkewedGaussianDistributionH\x00\x12=\n\tArbitrary\x18\x06 \x01(\x0b\x32(.grpc.SunSolve.API.ArbitraryDistributionH\x00\x42\n\n\x08\x46unction\"%\n\x14\x43onstantDistribution\x12\r\n\x05Value\x18\x01 \x01(\x01\"\xf1\x01\n\x14GaussianDistribution\x12\n\n\x02x0\x18\x01 \x01(\x01\x12\r\n\x05sigma\x18\x02 \x01(\x01\x12\x11\n\x04\x66max\x18\x03 \x01(\x01H\x00\x88\x01\x01\x12+\n\x1enumPointsInProbabilityFunction\x18\x04 \x01(\x05H\x01\x88\x01\x01\x12\x18\n\x0bxLowerLimit\x18\x05 \x01(\x01H\x02\x88\x01\x01\x12\x18\n\x0bxUpperLimit\x18\x06 \x01(\x01H\x03\x88\x01\x01\x42\x07\n\x05_fmaxB!\n\x1f_numPointsInProbabilityFunctionB\x0e\n\x0c_xLowerLimitB\x0e\n\x0c_xUpperLimit\"\xb4\x01\n\x1aSkewedGaussianDistribution\x12\r\n\x05\x61lpha\x18\x01 \x01(\x01\x12\x0c\n\x04zeta\x18\x02 \x01(\x01\x12\r\n\x05omega\x18\x03 \x01(\x01\x12\x11\n\x04\x66max\x18\x04 \x01(\x01H\x00\x88\x01\x01\x12+\n\x1enumPointsInProbabilityFunction\x18\x05 \x01(\x05H\x01\x88\x01\x01\x42\x07\n\x05_fmaxB!\n\x1f_numPointsInProbabilityFunction\"v\n\x13WeibullDistribution\x12\n\n\x02x0\x18\x01 \x01(\x01\x12\x0e\n\x06lambda\x18\x02 \x01(\x01\x12\t\n\x01k\x18\x03 \x01(\x01\x12 \n\x13hasPositivePolarity\x18\x04 \x01(\x08H\x00\x88\x01\x01\x42\x16\n\x14_hasPositivePolarity\"-\n\x15\x41rbitraryDistribution\x12\t\n\x01x\x18\x01 \x03(\x01\x12\t\n\x01y\x18\x02 \x03(\x01\"g\n\x1cYieldTimeSeriesSolarPosition\x12\x15\n\rZenithDegrees\x18\x01 \x01(\x02\x12\x16\n\x0e\x41zimuthDegrees\x18\x02 \x01(\x02\x12\x18\n\x10\x45levationDegrees\x18\x03 \x01(\x02\"\x94\x01\n\x16YieldTimeSeriesWeather\x12\x0b\n\x03GHI\x18\x01 \x01(\x02\x12\x0b\n\x03\x44HI\x18\x02 \x01(\x02\x12\x0b\n\x03\x44NI\x18\x03 \x01(\x02\x12\x17\n\x0f\x44iffuseFraction\x18\x04 \x01(\x02\x12\x13\n\x0b\x41mbientTemp\x18\x05 \x01(\x02\x12\n\n\x02WS\x18\x06 \x01(\x02\x12\n\n\x02WD\x18\x07 \x01(\x02\x12\r\n\x05\x43loud\x18\x08 \x01(\x02*#\n\x0cTrackingType\x12\n\n\x06NotSet\x10\x00\x12\x07\n\x03SAT\x10\x01*\xd3\x04\n\x11\x44istributionInput\x12\x07\n\x03GHI\x10\x00\x12\x13\n\x0f\x44iffuseFraction\x10\x01\x12\r\n\tWindSpeed\x10\x02\x12\x0f\n\x0bTemperature\x10\x03\x12\x0f\n\x0bModulePower\x10\x04\x12\x16\n\x12SpectralCorrection\x10\x05\x12\x10\n\x0cSoilingFront\x10\x06\x12\x0f\n\x0bSoilingRear\x10\x07\x12\x06\n\x02Uc\x10\x08\x12\x06\n\x02Uv\x10\t\x12\t\n\x05\x41lpha\x10\n\x12\x19\n\x15\x41nnualDegradationRate\x10\x0b\x12\x10\n\x0c\x41vailability\x10\x0c\x12\x11\n\rYieldModifier\x10\r\x12\x17\n\x13\x43ircumsolarFraction\x10\x0e\x12\x14\n\x10UndulatingGround\x10\x0f\x12\x13\n\x0f\x45xtraIrradiance\x10\x10\x12\x0f\n\x0b\x43urtailment\x10\x11\x12\x0c\n\x08\x44\x43Health\x10\x12\x12\x1e\n\x1aInverterToInverterMismatch\x10\x13\x12\x1a\n\x16StringToStringMismatch\x10\x14\x12\x1a\n\x16ModuleToModuleMismatch\x10\x15\x12\x16\n\x12\x43\x65llToCellMismatch\x10\x16\x12\x16\n\x12InverterEfficiency\x10\x17\x12%\n!ModulePowerTemperatureCoefficient\x10\x18\x12\n\n\x06\x41lbedo\x10\x19\x12\x1f\n\x1bRearStructuralShadingFactor\x10\x1a\x12\x1a\n\x16RearTransmissionFactor\x10\x1b*^\n\x10\x44istributionType\x12\x0c\n\x08\x43onstant\x10\x00\x12\x0c\n\x08Gaussian\x10\x01\x12\x0b\n\x07Weibull\x10\x02\x12\x12\n\x0eSkewedGaussian\x10\x03\x12\r\n\tArbitrary\x10\x04\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'uncertaintyMessages_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_TRACKINGTYPE']._serialized_start=7491
  _globals['_TRACKINGTYPE']._serialized_end=7526
  _globals['_DISTRIBUTIONINPUT']._serialized_start=7529
  _globals['_DISTRIBUTIONINPUT']._serialized_end=8124
  _globals['_DISTRIBUTIONTYPE']._serialized_start=8126
  _globals['_DISTRIBUTIONTYPE']._serialized_end=8220
  _globals['_UNCERTAINTYRESPONSE']._serialized_start=82
  _globals['_UNCERTAINTYRESPONSE']._serialized_end=332
  _globals['_UNCERTAINTYSIMULATIONRESULT']._serialized_start=335
  _globals['_UNCERTAINTYSIMULATIONRESULT']._serialized_end=506
  _globals['_UNCERTAINTYSUMMARY']._serialized_start=509
  _globals['_UNCERTAINTYSUMMARY']._serialized_end=694
  _globals['_UNCERTAINTYUSEDINPUTS']._serialized_start=697
  _globals['_UNCERTAINTYUSEDINPUTS']._serialized_end=1171
  _globals['_PVALUE']._serialized_start=1173
  _globals['_PVALUE']._serialized_end=1290
  _globals['_HISTOGRAM']._serialized_start=1292
  _globals['_HISTOGRAM']._serialized_end=1331
  _globals['_P50DEVIATION']._serialized_start=1333
  _globals['_P50DEVIATION']._serialized_end=1376
  _globals['_UNCERTAINTYREQUEST']._serialized_start=1379
  _globals['_UNCERTAINTYREQUEST']._serialized_end=2214
  _globals['_TMYTIMESTEPDATASET']._serialized_start=2216
  _globals['_TMYTIMESTEPDATASET']._serialized_end=2297
  _globals['_ACTUALTIMESTEPDATASET']._serialized_start=2299
  _globals['_ACTUALTIMESTEPDATASET']._serialized_end=2396
  _globals['_ACTUALTIMESTEPDATAFORYEAR']._serialized_start=2398
  _globals['_ACTUALTIMESTEPDATAFORYEAR']._serialized_end=2505
  _globals['_TIMESTEPDATA']._serialized_start=2508
  _globals['_TIMESTEPDATA']._serialized_end=3029
  _globals['_MODULEINFO']._serialized_start=3032
  _globals['_MODULEINFO']._serialized_end=3360
  _globals['_SYSTEMINFO']._serialized_start=3363
  _globals['_SYSTEMINFO']._serialized_end=3898
  _globals['_OPTICALSETTINGS']._serialized_start=3901
  _globals['_OPTICALSETTINGS']._serialized_end=4624
  _globals['_ELECTRICALSETTINGS']._serialized_start=4627
  _globals['_ELECTRICALSETTINGS']._serialized_end=5123
  _globals['_THERMALSETTINGS']._serialized_start=5125
  _globals['_THERMALSETTINGS']._serialized_end=5220
  _globals['_OPERATIONALSETTINGS']._serialized_start=5223
  _globals['_OPERATIONALSETTINGS']._serialized_end=5526
  _globals['_UNCERTAINTYSIMULATIONOPTIONS']._serialized_start=5529
  _globals['_UNCERTAINTYSIMULATIONOPTIONS']._serialized_end=5663
  _globals['_RESULTOPTIONS']._serialized_start=5666
  _globals['_RESULTOPTIONS']._serialized_end=5822
  _globals['_DISTRIBUTION']._serialized_start=5825
  _globals['_DISTRIBUTION']._serialized_end=6195
  _globals['_DISTRIBUTIONFUNCTION']._serialized_start=6198
  _globals['_DISTRIBUTIONFUNCTION']._serialized_end=6600
  _globals['_CONSTANTDISTRIBUTION']._serialized_start=6602
  _globals['_CONSTANTDISTRIBUTION']._serialized_end=6639
  _globals['_GAUSSIANDISTRIBUTION']._serialized_start=6642
  _globals['_GAUSSIANDISTRIBUTION']._serialized_end=6883
  _globals['_SKEWEDGAUSSIANDISTRIBUTION']._serialized_start=6886
  _globals['_SKEWEDGAUSSIANDISTRIBUTION']._serialized_end=7066
  _globals['_WEIBULLDISTRIBUTION']._serialized_start=7068
  _globals['_WEIBULLDISTRIBUTION']._serialized_end=7186
  _globals['_ARBITRARYDISTRIBUTION']._serialized_start=7188
  _globals['_ARBITRARYDISTRIBUTION']._serialized_end=7233
  _globals['_YIELDTIMESERIESSOLARPOSITION']._serialized_start=7235
  _globals['_YIELDTIMESERIESSOLARPOSITION']._serialized_end=7338
  _globals['_YIELDTIMESERIESWEATHER']._serialized_start=7341
  _globals['_YIELDTIMESERIESWEATHER']._serialized_end=7489
# @@protoc_insertion_point(module_scope)



import logging
import sys

import grpc

from ..grpcclient import uncertainty_grpc_client, uncertaintyMessages_pb2
from ..helpers.exception import AuthenticationError, P90ConnectionError, ServiceError
from ..helpers.request_helpers import format_number_for_display


class P90Client:
    """A gRPC client for the PV Lighthouse P90 analysis service."""

    def __init__(self, api_base_uri: str = "middleware.pvlighthouse.com.au", version: str = "latest-prod"):
        """Initialize the P90Client.
        Args:
            api_base_uri (str): The base URI for the API. Points to the production server by default
            version (str): The version of the API to use. Points to the latest production version by default

        Note:
            The API base URI and version can be overridden for testing or development purposes. This can also
            pointing specifically to a previous version of the service (eg: 7.20.0) should the client become
            incompatible with the latest version and while a later version of the client is not available.
        """
        try:
            self.channel = grpc.secure_channel(api_base_uri, grpc.ssl_channel_credentials())
            self.client = uncertainty_grpc_client.UncertaintyStub(self.channel)
            self.metadata = [("version", version)]
            if self.is_dev_version(version):
                self.metadata.append(("x-cloudfront-env", "dev"))
            else:
                self.metadata.append(("x-cloudfront-env", "prod"))
            logging.info("Connected to P90 service at %s", api_base_uri)
        except Exception as e:
            raise P90ConnectionError(f"Failed to connect to P90 service at {api_base_uri}: {e}") from e

    def is_dev_version(self, version: str) -> bool:
        """Determine if the provided version string indicates a development version."""
        if "latest-dev" == version:
            return True

        version_split = version.split(".")
        if len(version_split) >= 4:
            return True

        return False

    def _handle_grpc_error(self, error: grpc.RpcError, timeout: float | None) -> None:
        """Handle specific gRPC errors and raise appropriate exceptions."""
        error_handlers = {
            grpc.StatusCode.UNAUTHENTICATED: lambda: AuthenticationError("Authentication failed. Please check your credentials."),
            grpc.StatusCode.PERMISSION_DENIED: lambda: AuthenticationError(
                "Permission denied. Please check your account permissions and contact support@pvlighthouse.com.au."
            ),
            grpc.StatusCode.RESOURCE_EXHAUSTED: lambda: ServiceError(f"{error.details()}. Please contact support@pvlighthouse.com.au."),
            grpc.StatusCode.UNIMPLEMENTED: lambda: ServiceError("Service endpoint not implemented or not found."),
            grpc.StatusCode.UNAVAILABLE: lambda: P90ConnectionError("Service is currently unavailable. Please try again later."),
            grpc.StatusCode.DEADLINE_EXCEEDED: lambda: TimeoutError(f"Request timed out after {timeout} seconds."),
            grpc.StatusCode.INVALID_ARGUMENT: lambda: ValueError(f"Invalid request parameters: {error.details()}"),
            grpc.StatusCode.CANCELLED: lambda: P90ConnectionError("Request was cancelled. This may happen if the stream is closed unexpectedly."),
            grpc.StatusCode.ABORTED: lambda: P90ConnectionError("Request was aborted by the server. The stream may have been interrupted."),
        }

        if handler := error_handlers.get(error.code()):
            raise handler() from error
        raise ServiceError(f"Service error ({error.code().name}): {error.details()}") from error

    def _log_inputs_used(self, used_inputs: uncertaintyMessages_pb2.UncertaintyUsedInputs) -> None:
        """Log comprehensive details about the inputs used in the P90 analysis."""
        logging.debug("=== P90 Analysis Inputs Used ===")

        # Module Information
        if used_inputs.HasField("Module"):
            module = used_inputs.Module
            logging.debug(
                "Module: Length=%.3fm, Width=%.3fm, PowerRatingAtSTC=%.1fW, CellToCellMismatch=%.4f, PowerTempCoeff=%.6f, Bifaciality=%.3f",
                module.LengthInM,
                module.WidthInM,
                module.PowerRatingAtSTCInW,
                module.CellToCellMismatch,
                module.ModulePowerTemperatureCoefficient,
                module.Bifaciality,
            )
        else:
            logging.debug("Module: No module information provided")

        # System Information
        if used_inputs.HasField("System"):
            system = used_inputs.System
            logging.debug(
                "System: ModulesPerString=%d, StringsPerInverter=%d, NumberOfInverters=%d",
                system.ModulesPerString,
                system.StringsPerInverter,
                system.NumberOfInverters,
            )
            logging.debug(
                "System Geometry: RowPitch=%.2fm, ModuleAzimuth=%.1f°, ModuleHeight=%.3fm, FallbackModuleTilt=%.1f°",
                system.RowPitchInM,
                system.ModuleAzimuthInDegrees,
                system.ModuleHeight,
                system.FallbackModuleTiltInDegrees,
            )
            logging.debug(
                "System Tracking: ModuleTracking=%s, TiltLimit=%.1f°",
                system.ModuleTracking,
                system.TiltLimitInDegrees,
            )
        else:
            logging.debug("System: No system information provided")

        # Electrical System Information
        if used_inputs.HasField("Electrical"):
            electrical = used_inputs.Electrical
            logging.debug(
                "Electrical Efficiencies: InverterEff=%.4f, ModuleToModuleMismatch=%.4f, StringWiringLoss=%.4f, InverterClipping=%s",
                electrical.InverterEfficiency,
                electrical.ModuleToModuleMismatch,
                electrical.StringWiringLoss,
                format_number_for_display(electrical.InverterClipping),
            )
            logging.debug(
                "Electrical Losses: MaxPowerTracking=%.4f, InverterWiring=%.4f, StringToStringMismatch=%.4f, InverterToInverterMismatch=%.4f",
                electrical.MaxPowerTrackingLoss,
                electrical.InverterWiringLoss,
                electrical.StringToStringMismatch,
                electrical.InverterToInverterMismatch,
            )
        else:
            logging.debug("Electrical settings: No electrical information provided")

        # Optical System Information
        if used_inputs.HasField("Optical"):
            optical = used_inputs.Optical
            logging.debug(
                "Optical Multipliers: BeamFront=%.3f, BeamRear=%.3f, IsotropicFront=%.3f, IsotropicRear=%.3f",
                optical.BeamMultiplierFront,
                optical.BeamMultiplierRear,
                optical.IsotropicMultiplierFront,
                optical.IsotropicMultiplierRear,
            )
            logging.debug(
                "Optical Fallbacks: Albedo=%.3f, SoilingFront=%.4f, SoilingRear=%.4f, SpectralCorrection=%.4f, ExtraIrradiance=%.4f, "
                "RearTransmissionFactor=%.3f, RearStructuralShadingFactor=%.3f",
                optical.FallbackAlbedo,
                optical.FallbackSoilingFront,
                optical.FallbackSoilingRear,
                optical.FallbackSpectralCorrection,
                optical.ExtraIrradiance,
                optical.FallbackRearTransmissionFactor,
                optical.FallbackRearStructuralShadingFactor,
            )
        else:
            logging.debug("Optical settings: No optical information provided")

        # Thermal system Information
        if used_inputs.HasField("Thermal"):
            thermal = used_inputs.Thermal
            logging.debug(
                "Thermal: Uc=%.2f, Uv=%.2f, Alpha=%.6f",
                thermal.Uc,
                thermal.Uv,
                thermal.Alpha,
            )
        else:
            logging.debug("Thermal settings: No thermal information provided")

        # Operational System Information
        if used_inputs.HasField("Operation"):
            operational = used_inputs.Operation
            logging.debug(
                "Operational: AnnualDegradationRate=%.4f, Curtailment=%.4f, Availability=%.4f, DCHealth=%.4f, UndulatingGround=%f, YieldModifier=%.4f",
                operational.AnnualDegradationRate,
                operational.Curtailment,
                operational.Availability,
                operational.DCHealth,
                operational.UndulatingGround,
                operational.YieldModifier,
            )
        else:
            logging.debug("Operational settings: No operational information provided")

        # Simulation Options
        if used_inputs.HasField("SimulationOptions"):
            sim_opts = used_inputs.SimulationOptions
            logging.debug(
                "Simulation: %d years, %d simulations",
                sim_opts.NumberOfYears,
                sim_opts.NumberOfSimulations,
            )
        else:
            logging.debug("Simulation: No simulation options provided")

        # Result Options Information
        if used_inputs.HasField("ResultOptions"):
            result_options = used_inputs.ResultOptions
            p_values_str = ",".join(map(str, result_options.PValues)) if result_options.PValues else "None"
            logging.debug(
                "Result Options: BinMin=%.2f, BinDelta=%.2f, PValues=[%s]",
                result_options.BinMin,
                result_options.BinDelta,
                p_values_str,
            )
        else:
            logging.debug("Result options: No result options provided")

        logging.debug("=== End Inputs Used ===")

    def send_request(
        self,
        request: uncertaintyMessages_pb2.UncertaintyRequest,
        call_credentials: grpc.CallCredentials,
        timeout: float | None = 120.0,
    ) -> tuple[uncertaintyMessages_pb2.UncertaintySummary | None, uncertaintyMessages_pb2.UncertaintyUsedInputs | None]:
        """Send a P90 analysis request and return the summary results and used inputs."""
        if not request.ActualYearlyDataSet.TimeStepDataPoints and not request.TMYDataSet.TimeStepDataPoints:
            raise ValueError("Request must contain actual yearly data or TMY data")

        summary = None
        used_inputs = None

        try:
            logging.info("Starting P90 analysis request")
            response_stream = self.client.UncertaintyAnalysis(request, metadata=self.metadata, credentials=call_credentials, timeout=timeout)

            for response in response_stream:
                received: uncertaintyMessages_pb2.UncertaintyResponse = response

                if received.HasField("InputsUsed"):
                    used_inputs = received.InputsUsed
                    self._log_inputs_used(used_inputs)

                # Overwrite the same line with progress
                sys.stdout.write(f"\rProgress: {received.pctComplete * 100:.2f}%")
                sys.stdout.flush()

                # Check for summary in response
                if received.HasField("Summary"):
                    summary = received.Summary
                    print("\n")
                    logging.info("Received P90 analysis summary")
                    print(f"Analysis complete! Summary contains {len(summary.YearlyPValue)} yearly P-values")

        except grpc.RpcError as e:
            print("\n")
            logging.error("gRPC error during P90 analysis: %s", e)
            self._handle_grpc_error(e, timeout)
        except (ValueError, TypeError, AttributeError) as e:
            print("\n")
            logging.error("Unexpected error during P90 analysis: %s", e)
            raise P90ConnectionError(f"Unexpected error during P90 analysis: {e}") from e

        if not summary or not summary.YearlyPValue:
            print("\n")
            logging.warning("No results received from P90 analysis")

        return summary, used_inputs

    def close(self) -> None:
        """Close the gRPC channel and clean up resources."""
        try:
            self.channel.close()
            logging.info("P90 client connection closed")
        except Exception as e:
            logging.warning("Error closing P90 client: %s", e)
            raise P90ConnectionError(f"Unexpected error during P90 analysis: {e}") from e

    def __enter__(self):
        """Context manager entry."""
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """Context manager exit."""
        self.close()



import datetime

from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf import duration_pb2 as _duration_pb2
from google.protobuf import struct_pb2 as _struct_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from collections.abc import Iterable as _Iterable, Mapping as _Mapping
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class DataSourceEnum(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    UPLOAD: _ClassVar[DataSourceEnum]
    PVGIS: _ClassVar[DataSourceEnum]

class ObservationTypeEnum(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    TMY: _ClassVar[ObservationTypeEnum]
    OBSERVED: _ClassVar[ObservationTypeEnum]

class ObservationPeriodEnum(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    Deprecated_ZenithAzimuthSpecified: _ClassVar[ObservationPeriodEnum]
    Instantaneous: _ClassVar[ObservationPeriodEnum]
    EndOfHour: _ClassVar[ObservationPeriodEnum]
    StartOfHour: _ClassVar[ObservationPeriodEnum]
    MiddleOfHour: _ClassVar[ObservationPeriodEnum]
UPLOAD: DataSourceEnum
PVGIS: DataSourceEnum
TMY: ObservationTypeEnum
OBSERVED: ObservationTypeEnum
Deprecated_ZenithAzimuthSpecified: ObservationPeriodEnum
Instantaneous: ObservationPeriodEnum
EndOfHour: ObservationPeriodEnum
StartOfHour: ObservationPeriodEnum
MiddleOfHour: ObservationPeriodEnum

class WeatherSetting(_message.Message):
    __slots__ = ("weatherRows", "Latitude", "Longitude", "Elevation_Metres", "StartDate", "EndDate", "ObservationType", "DataSource", "HorizonShadowsIncluded", "RadiationDB", "MeteoDB", "HorizonDB", "LocalStandardTimeOffset_Minutes", "LocalTimeZone_IANA", "ObservationPeriod", "versionNumber", "ObservationPeriodDuration")
    WEATHERROWS_FIELD_NUMBER: _ClassVar[int]
    LATITUDE_FIELD_NUMBER: _ClassVar[int]
    LONGITUDE_FIELD_NUMBER: _ClassVar[int]
    ELEVATION_METRES_FIELD_NUMBER: _ClassVar[int]
    STARTDATE_FIELD_NUMBER: _ClassVar[int]
    ENDDATE_FIELD_NUMBER: _ClassVar[int]
    OBSERVATIONTYPE_FIELD_NUMBER: _ClassVar[int]
    DATASOURCE_FIELD_NUMBER: _ClassVar[int]
    HORIZONSHADOWSINCLUDED_FIELD_NUMBER: _ClassVar[int]
    RADIATIONDB_FIELD_NUMBER: _ClassVar[int]
    METEODB_FIELD_NUMBER: _ClassVar[int]
    HORIZONDB_FIELD_NUMBER: _ClassVar[int]
    LOCALSTANDARDTIMEOFFSET_MINUTES_FIELD_NUMBER: _ClassVar[int]
    LOCALTIMEZONE_IANA_FIELD_NUMBER: _ClassVar[int]
    OBSERVATIONPERIOD_FIELD_NUMBER: _ClassVar[int]
    VERSIONNUMBER_FIELD_NUMBER: _ClassVar[int]
    OBSERVATIONPERIODDURATION_FIELD_NUMBER: _ClassVar[int]
    weatherRows: _containers.RepeatedCompositeFieldContainer[WeatherRow]
    Latitude: float
    Longitude: float
    Elevation_Metres: float
    StartDate: _timestamp_pb2.Timestamp
    EndDate: _timestamp_pb2.Timestamp
    ObservationType: ObservationTypeEnum
    DataSource: DataSourceEnum
    HorizonShadowsIncluded: bool
    RadiationDB: str
    MeteoDB: str
    HorizonDB: str
    LocalStandardTimeOffset_Minutes: NullableInt32
    LocalTimeZone_IANA: str
    ObservationPeriod: ObservationPeriodEnum
    versionNumber: int
    ObservationPeriodDuration: _duration_pb2.Duration
    def __init__(self, weatherRows: _Optional[_Iterable[_Union[WeatherRow, _Mapping]]] = ..., Latitude: _Optional[float] = ..., Longitude: _Optional[float] = ..., Elevation_Metres: _Optional[float] = ..., StartDate: _Optional[_Union[datetime.datetime, _timestamp_pb2.Timestamp, _Mapping]] = ..., EndDate: _Optional[_Union[datetime.datetime, _timestamp_pb2.Timestamp, _Mapping]] = ..., ObservationType: _Optional[_Union[ObservationTypeEnum, str]] = ..., DataSource: _Optional[_Union[DataSourceEnum, str]] = ..., HorizonShadowsIncluded: bool = ..., RadiationDB: _Optional[str] = ..., MeteoDB: _Optional[str] = ..., HorizonDB: _Optional[str] = ..., LocalStandardTimeOffset_Minutes: _Optional[_Union[NullableInt32, _Mapping]] = ..., LocalTimeZone_IANA: _Optional[str] = ..., ObservationPeriod: _Optional[_Union[ObservationPeriodEnum, str]] = ..., versionNumber: _Optional[int] = ..., ObservationPeriodDuration: _Optional[_Union[datetime.timedelta, _duration_pb2.Duration, _Mapping]] = ...) -> None: ...

class WeatherRow(_message.Message):
    __slots__ = ("ObservationTime_UTC", "GlobalHorizontalIrradiance_WattsPerMetreSquared", "DirectNormalIrradiance_WattsPerMetreSquared", "DiffuseHorizontalIrradiance_WattsPerMetreSquared", "SurfaceAirPressure_Pascals", "DryBulbTemperature_DegreesCeliusAt2Metres", "WindVelocity_MetresPerSecondAt10Metres", "WindDirection_DegreesClockwiseFromNorthAt10Metres", "RelativeHumidity_Percent", "SurfaceInfraredIrradiance_WattsPerMeterSquared", "PrecipitableWaterVapourCM", "SolarZenith", "SolarAzimuth", "Ozone_AtmCm", "Turbidity", "FarFieldAlbedo_Percent")
    OBSERVATIONTIME_UTC_FIELD_NUMBER: _ClassVar[int]
    GLOBALHORIZONTALIRRADIANCE_WATTSPERMETRESQUARED_FIELD_NUMBER: _ClassVar[int]
    DIRECTNORMALIRRADIANCE_WATTSPERMETRESQUARED_FIELD_NUMBER: _ClassVar[int]
    DIFFUSEHORIZONTALIRRADIANCE_WATTSPERMETRESQUARED_FIELD_NUMBER: _ClassVar[int]
    SURFACEAIRPRESSURE_PASCALS_FIELD_NUMBER: _ClassVar[int]
    DRYBULBTEMPERATURE_DEGREESCELIUSAT2METRES_FIELD_NUMBER: _ClassVar[int]
    WINDVELOCITY_METRESPERSECONDAT10METRES_FIELD_NUMBER: _ClassVar[int]
    WINDDIRECTION_DEGREESCLOCKWISEFROMNORTHAT10METRES_FIELD_NUMBER: _ClassVar[int]
    RELATIVEHUMIDITY_PERCENT_FIELD_NUMBER: _ClassVar[int]
    SURFACEINFRAREDIRRADIANCE_WATTSPERMETERSQUARED_FIELD_NUMBER: _ClassVar[int]
    PRECIPITABLEWATERVAPOURCM_FIELD_NUMBER: _ClassVar[int]
    SOLARZENITH_FIELD_NUMBER: _ClassVar[int]
    SOLARAZIMUTH_FIELD_NUMBER: _ClassVar[int]
    OZONE_ATMCM_FIELD_NUMBER: _ClassVar[int]
    TURBIDITY_FIELD_NUMBER: _ClassVar[int]
    FARFIELDALBEDO_PERCENT_FIELD_NUMBER: _ClassVar[int]
    ObservationTime_UTC: _timestamp_pb2.Timestamp
    GlobalHorizontalIrradiance_WattsPerMetreSquared: NullableFloat
    DirectNormalIrradiance_WattsPerMetreSquared: NullableFloat
    DiffuseHorizontalIrradiance_WattsPerMetreSquared: NullableFloat
    SurfaceAirPressure_Pascals: NullableFloat
    DryBulbTemperature_DegreesCeliusAt2Metres: NullableFloat
    WindVelocity_MetresPerSecondAt10Metres: NullableFloat
    WindDirection_DegreesClockwiseFromNorthAt10Metres: NullableFloat
    RelativeHumidity_Percent: NullableFloat
    SurfaceInfraredIrradiance_WattsPerMeterSquared: NullableFloat
    PrecipitableWaterVapourCM: NullableFloat
    SolarZenith: NullableFloat
    SolarAzimuth: NullableFloat
    Ozone_AtmCm: NullableFloat
    Turbidity: NullableFloat
    FarFieldAlbedo_Percent: NullableFloat
    def __init__(self, ObservationTime_UTC: _Optional[_Union[datetime.datetime, _timestamp_pb2.Timestamp, _Mapping]] = ..., GlobalHorizontalIrradiance_WattsPerMetreSquared: _Optional[_Union[NullableFloat, _Mapping]] = ..., DirectNormalIrradiance_WattsPerMetreSquared: _Optional[_Union[NullableFloat, _Mapping]] = ..., DiffuseHorizontalIrradiance_WattsPerMetreSquared: _Optional[_Union[NullableFloat, _Mapping]] = ..., SurfaceAirPressure_Pascals: _Optional[_Union[NullableFloat, _Mapping]] = ..., DryBulbTemperature_DegreesCeliusAt2Metres: _Optional[_Union[NullableFloat, _Mapping]] = ..., WindVelocity_MetresPerSecondAt10Metres: _Optional[_Union[NullableFloat, _Mapping]] = ..., WindDirection_DegreesClockwiseFromNorthAt10Metres: _Optional[_Union[NullableFloat, _Mapping]] = ..., RelativeHumidity_Percent: _Optional[_Union[NullableFloat, _Mapping]] = ..., SurfaceInfraredIrradiance_WattsPerMeterSquared: _Optional[_Union[NullableFloat, _Mapping]] = ..., PrecipitableWaterVapourCM: _Optional[_Union[NullableFloat, _Mapping]] = ..., SolarZenith: _Optional[_Union[NullableFloat, _Mapping]] = ..., SolarAzimuth: _Optional[_Union[NullableFloat, _Mapping]] = ..., Ozone_AtmCm: _Optional[_Union[NullableFloat, _Mapping]] = ..., Turbidity: _Optional[_Union[NullableFloat, _Mapping]] = ..., FarFieldAlbedo_Percent: _Optional[_Union[NullableFloat, _Mapping]] = ...) -> None: ...

class NullableFloat(_message.Message):
    __slots__ = ("null", "data")
    NULL_FIELD_NUMBER: _ClassVar[int]
    DATA_FIELD_NUMBER: _ClassVar[int]
    null: _struct_pb2.NullValue
    data: float
    def __init__(self, null: _Optional[_Union[_struct_pb2.NullValue, str]] = ..., data: _Optional[float] = ...) -> None: ...

class NullableInt32(_message.Message):
    __slots__ = ("null", "data")
    NULL_FIELD_NUMBER: _ClassVar[int]
    DATA_FIELD_NUMBER: _ClassVar[int]
    null: _struct_pb2.NullValue
    data: int
    def __init__(self, null: _Optional[_Union[_struct_pb2.NullValue, str]] = ..., data: _Optional[int] = ...) -> None: ...



"""
PV Lighthouse P90 Analysis Client

A Python client library for connecting to the PV Lighthouse P90 analysis service via gRPC.
"""

from .client.p90_client import P90Client
from .helpers.exception import AuthenticationError, P90ClientError, P90ConnectionError, ServiceError

# Version is set dynamically during build process
try:
    from .__version__ import __version__
except ImportError:
    # Fallback for development environment
    __version__ = "dev"

__all__ = [
    "P90Client",
    "P90ClientError",
    "AuthenticationError",
    "P90ConnectionError",
    "ServiceError",
    "__version__",
]
